// EAllocatorBin.h: interface for the EAllocatorBin class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EALLOCATORBIN_H__FD31CFC4_BB54_11D2_91EF_004005446EE6__INCLUDED_)
#define AFX_EALLOCATORBIN_H__FD31CFC4_BB54_11D2_91EF_004005446EE6__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "EAllocator.h"

template< class TElm,class EParentAllocator = EAllocatorT< TElm > >
class EAllocatorBinT : public EParentAllocator
{
public:
  EAllocatorBinT() {}
  virtual ~EAllocatorBinT() {}

  /*! Initializes an object of the array.
   *
   * @param nIndex Zero based index of the object to initialize.
   */
  virtual void construct( SL nIndex ) {}

  /*! Initializes a sequence of object in the array.
   *
   * @param nIndex Zero based index of the first object to initialize.
   * @param nLen Number of object to initialize.
   */
  virtual void construct( SL nIndex,
                          SL nLen ) {}

  /*! Destructs an object of the array.
   * @param nIndex Zero based index of the object to destruct.
   */
  virtual void destruct( SL nIndex ) {}

  /*! Destructs a sequence list of object in the array.
   * @param nIndex Zero based index of the first object to destruct. 
   * @param nLen Number of object to destruct. May be 0.
   */
  virtual void destruct( SL nIndex, 
                         SL nLen ) {}

  /*! Copy a sequence of object from this array to another one.
   * @param nIndexSource Zero based index of the first object of the sequence to copy.
   * @param nLen Number of object in the sequence. May be zero.
   * @param nIndexDestination Zero based index of destination array where the sequence
   *                          is copied to.
   * @param allocatorDest Destination array.
   */
  virtual void copyTo( SL nIndexSource, 
                       SL nLen,
                       SL nIndexDestination,
                       EAllocatorT<TElm> &allocatorDest ) const;

  /*! Copy a sequence of object from a standard C array to this array.
   * @param arraySource Pointer on the first element of the source array to copy.
   * @param nLen Number of objects in the sequence. May be zero.
   * @param nIndexDestination Zero based index where the sequence is copied to.
   */
  virtual void copyFrom( const TElm *arraySource, 
                         SL nLen,
                         SL nIndexDestination );

  /*! Move a sequence of object inside the array.
   * @param nIndex Zero based index of the first element to move.
   * @param nLen Number of elements to move.
   * @param nIndexDestination Zero based index where the first element is moved to.
   */
  virtual void copyToThis( SL nIndex,
                           SL nLen,
                           SL nIndexDestination );
  /**@shapeType DependencyLink*/
  /*#  EAllocatorT lnkUnnamed*/
};



template<class TElm,class EParentAllocator>
void 
EAllocatorBinT<TElm,EParentAllocator>::copyTo( SL nIndexSource, 
                                               SL nLen,
                                               SL nIndexDestination,
                                               EAllocatorT<TElm> &allocatorDest ) const
{
  EASSERT( nIndexSource <= count() );
  EASSERT( nLen >= 0 );
  EASSERT( nIndexSource + nLen <= count() );
  EASSERT( nIndexDestination <= allocatorDest.count() );
  EASSERT( nIndexDestination + nLen <= allocatorDest.count() );

  if ( !nLen )
    return;

  memcpy( &allocatorDest.at( nIndexDestination ), 
          &at( nIndexSource ),
          sizeof( TElm ) * nLen );
}


template<class TElm,class EParentAllocator>
void 
EAllocatorBinT<TElm,EParentAllocator>::copyFrom( const TElm *arraySource, 
                                                 SL nLen,
                                                 SL nIndexDestination )
{
  EASSERT( nIndexDestination >= 0 );
  EASSERT( nLen >= 0 );
  EASSERT( nIndexDestination + nLen <= count() );

  if ( !nLen )
    return;

  EASSERT( arraySource != NULL );

  memcpy( &at( nIndexDestination ),
          arraySource,
          sizeof( TElm ) * nLen );
}


template<class TElm,class EParentAllocator>
void 
EAllocatorBinT<TElm,EParentAllocator>::copyToThis( SL nIndex,
                                                   SL nLen,
                                                   SL nIndexDestination )
{
  EASSERT( nIndex >= 0 );
  EASSERT( nIndex + nLen <= count() );
  EASSERT( nIndexDestination >= 0 );
  EASSERT( nIndexDestination + nLen <= count() );

  if ( nIndex == nIndexDestination  ||
       !nLen )
    return;

  memmove( &at( nIndexDestination ),
           &at( nIndex ),
           nLen * sizeof( TElm ) );
}

#endif // !defined(AFX_EALLOCATORBIN_H__FD31CFC4_BB54_11D2_91EF_004005446EE6__INCLUDED_)
