// EAutoPtrT.h: interface for the EAutoPtrT class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EAUTOPTRT_H__49E23C52_834A_11D3_BEC1_0040053AD648__INCLUDED_)
#define AFX_EAUTOPTRT_H__49E23C52_834A_11D3_BEC1_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

template<class Type>
class EAutoPtrT
{
public:
  typedef EAutoPtrT<Type> ThisType;

  /** Construct a auto pointer.
   */
	explicit EAutoPtrT( Type *pObject =NULL );

  /*! Takes the ownership of a pointer from another auto pointer.
   * @param ptrToTake Auto pointer which this object take the ownership.
   */
  EAutoPtrT( ThisType &ptrToTake );

  /** Destructor.
   *
   * If the object is attached, it is destroyed.
   */
	virtual ~EAutoPtrT();

  /** Detach the object.
   * The auto-pointer lose the ownership of the pointed object. On return,
   * the auto-pointer is \c NULL.
   * @return Pointer on the object that was attached. May be \c NULL.
   */
  Type *detach();

  /*! Destroys the attached object.
   */
  void reset();

  /*! Gets the pointer on the object.
   * @return Pointer on the pointed object. May be \c NULL.
   */
  Type *get() const;

  Type *operator->() const;

  operator Type *() const;

  /*! Take the ownership of another auto pointer.
   * @param TypeCastFrom Type of auto-pointer to copy.
   * @param ptrToTake Auto pointer which this object take the ownership.
   * @return Reference on this object.
   */
#if 0   // Not supported by VC 5.0 ++
  template<class TypeCastFrom>
  ThisType &operator =( EAutoPtrT<TypeCastFrom> &ptrToTake )
  {
    if ( _bIsOwned  &&  _pObject )
      delete _pObject;
    _pObject = ptrToTake._pObject;
    ptrToTake._pObject = NULL;
    _bIsOwned = ptrToTake._bIsOwned;
    ptrToTake._bIsOwned = false;
    return *this;
  }
#else
  ThisType &operator =( ThisType &ptrToTake )
  {
    if ( _bIsOwned  &&  _pObject )
      delete _pObject;
    _pObject = ptrToTake._pObject;
    ptrToTake._pObject = NULL;
    _bIsOwned = ptrToTake._bIsOwned;
    ptrToTake._bIsOwned = false;
    return *this;
  }
#endif

private:
  /** Pointer on the object. May be \c NULL.
   */
  Type *_pObject;

  /*! Indicates if the pointer is owned.
   */
  bool _bIsOwned;
};



template<class Type>
inline
EAutoPtrT<Type>::EAutoPtrT( Type *pObject ) :
    _pObject( pObject ),
    _bIsOwned( true )
{
}


template<class Type>
inline
EAutoPtrT<Type>::EAutoPtrT( ThisType &ptrToTake ) :
    _pObject( ptrToTake._pObject ),
    _bIsOwned( true )
{
  ptrTotake._bIsOwned = false;
  ptrToTake._pObject = NULL;
}


template<class Type>
inline
EAutoPtrT<Type>::~EAutoPtrT()
{
  if ( _bIsOwned  &&  _pObject )
    delete _pObject;
}


template<class Type>
inline Type *
EAutoPtrT<Type>::detach()
{
  Type *pObject = _pObject;
  _pObject = NULL;
  _bIsOwned = false;
  return pObject;
}


template<class Type>
inline Type *
EAutoPtrT<Type>::operator->() const
{
  EASSERT( _pObject );
  return _pObject;
}


template<class Type>
inline
EAutoPtrT<Type>::operator Type *() const
{
  return _pObject;
}


template<class Type>
inline Type *
EAutoPtrT<Type>::get() const
{
  return _pObject;
}


template<class Type>
void 
EAutoPtrT<Type>::reset()
{
  Type *pObj = detach();
  if ( pObj )
    delete pObj;
}



#endif // !defined(AFX_EAUTOPTRT_H__49E23C52_834A_11D3_BEC1_0040053AD648__INCLUDED_)
