// EBaseHashSetIteratorT.h: interface for the EBaseHashSetIteratorT class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EBASEHASHSETITERATORT_H__992AFCE9_3D03_11D3_BE1D_0040053AD648__INCLUDED_)
#define AFX_EBASEHASHSETITERATORT_H__992AFCE9_3D03_11D3_BE1D_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000



#include "EIteratorT.h"

template<class Key, class KeyArg>
class EBaseHashSetIteratorT : public EIteratorT<Key>
{
public:
  EBaseHashSetIteratorT( const EBaseHashSetT<Key,KeyArg> &hashTable );
  EBaseHashSetIteratorT();

  void attach(  const EBaseHashSetT<Key,KeyArg> &hashTable );

  /** Set the iterator at the beginning of the table.
   * If there is no element in the table then isEnd() will return true.
   */
  void begin();

  /** Test if the iterator is beyond the last association.
   */
  bool isEnd() const;

  /** Remove the association at the current position of the iterator.
   */
  void remove();

  /** Move the iterator to the next association.
   */
  void next();

  /** Get the item at the current position of the iterator.
   * @return item at the current position of the iterator.
   */
  Key& item();

  /** Get the item at the current position of the iterator.
   * @return item at the current position of the iterator.
   */
  const Key& item() const;

  /** Get the key at the current position of the iterator.
   * @return key at the current position of the iterator.
   */
  const Key& key() const;

  /*! Get the numbers of items.
   * @return Number of items.
   */
  SL count() const;

private:
  /** Search the next index with an association.
   * The search begins at the index _nIndex +1.
   */
  void searchNextIndex();

  /// Hash table the iterator operate on.
  const EBaseHashSetT<Key,KeyArg> *_pHashTable;

  /// Current index in association table.
  SL _nIndex;

  /// Current Association for the list at nIndex.
  const typename EBaseHashSetT<Key,KeyArg>::ELCAsso *_pAsso;
};



// Template implementation:
// #####################Last change: 28/03/98 20h01mn#########################

template<class Key, class KeyArg>
EBaseHashSetIteratorT<Key,KeyArg>::EBaseHashSetIteratorT(
    const EBaseHashSetT<Key,KeyArg> &hashTable )
{
  _pHashTable = &hashTable;
  _nIndex = 0;
  _pAsso = ENIL;
  begin();
}


template<class Key, class KeyArg>
EBaseHashSetIteratorT<Key,KeyArg>::EBaseHashSetIteratorT()
{
  _pHashTable = NULL;
  _nIndex = 0;
  _pAsso = ENIL;
}


template<class Key, class KeyArg>
void
EBaseHashSetIteratorT<Key,KeyArg>::attach(
    const EBaseHashSetT<Key,KeyArg> &hashTable )
{
  EASSERT( pHashTable );
  _pHashTable = pHashTable;
  _nIndex = 0;
  _pAsso = ENIL;
  begin();
}

template<class Key, class KeyArg>
void EBaseHashSetIteratorT<Key,KeyArg>::begin()
{
  EASSERT( _pHashTable );
  _nIndex = -1;
  searchNextIndex();
}

template<class Key, class KeyArg>
bool EBaseHashSetIteratorT<Key,KeyArg>::isEnd() const
{
  EASSERT( _pHashTable );
  return _pAsso == ENIL;
}


template<class Key, class KeyArg>
void EBaseHashSetIteratorT<Key,KeyArg>::remove()
{
  EASSERT( _pHashTable );
  if ( isEnd() )
    return;

  const EBaseHashSetT<Key,KeyArg>::ELCAsso *pAsso =
      _pHashTable->_aAsso[ _nIndex ];
  const EBaseHashSetT<Key,KeyArg>::ELCAsso **pPrevAsso =
      &_pHashTable->_aAsso[ _nIndex ];

  while ( pAsso != _pAsso )     // search previous association
  {
    EASSERT( pAsso );
    pPrevAsso = &pAsso->_pNext;
    pAsso = pAsso->_pNext;
  }

  _pHashTable->doFreeAsso( _pAsso->_key );

  *pPrevAsso = _pAsso->_pNext;       // unchain
  delete _pAsso;
  _pAsso = *pPrevAsso;
  _pHashTable->_nNbAsso--;

  if ( !_pAsso )                  // if no more in the list for this index
  {                               // then search next.
    searchNextIndex();
  }
}


template<class Key, class KeyArg>
void EBaseHashSetIteratorT<Key,KeyArg>::next()
{
  EASSERT( _pHashTable );
  if ( _pAsso )
  {
    _pAsso = _pAsso->_pNext;
    if ( !_pAsso )          // end of current index reached ?
      searchNextIndex();
  }
}


template<class Key, class KeyArg>
void EBaseHashSetIteratorT<Key,KeyArg>::searchNextIndex()
{
  EASSERT( _pHashTable );
  SL nSize = _pHashTable->size();
  _nIndex++;
  for ( ;_nIndex < nSize; _nIndex++ )
  {
    if ( _pHashTable->_aAsso[ _nIndex ] )
    {
      _pAsso = _pHashTable->_aAsso[ _nIndex ];
      return;
    }
  }
  _pAsso = ENIL;
}

template<class Key, class KeyArg>
const Key& EBaseHashSetIteratorT<Key,KeyArg>::item() const
{
  EASSERT( _pHashTable );
  EASSERT( _pAsso );
  return _pAsso->_key;
}

template<class Key, class KeyArg>
Key& EBaseHashSetIteratorT<Key,KeyArg>::item()
{
  EASSERT( _pHashTable );
  EASSERT( _pAsso );
  return *(Key *)&_pAsso->_key;
}


template<class Key, class KeyArg>
inline SL 
EBaseHashSetIteratorT<Key,KeyArg>::count() const
{
  EASSERT( _pHashTable );
  return _pHashTable->count();
}


#endif // !defined(AFX_EBASEHASHSETITERATORT_H__992AFCE9_3D03_11D3_BE1D_0040053AD648__INCLUDED_)
