// ###########################################################################
// File: ECallback2RT.h
// (c)Copyright 03/1999, Gaia Creation, Baptiste Lepilleur
// ###########################################################################
#if !defined(AFX_ECALLBACK2RT_H__B78AA25B_D476_11D2_BD55_0040053AD648__INCLUDED_)
#define AFX_ECALLBACK2RT_H__B78AA25B_D476_11D2_BD55_0040053AD648__INCLUDED_


#include "EBaseCallback.h"

/*! 
 * \brief This class provides a callback without parameters and return value.
 */
template<class TP1,class TP2,class ReturnType>
class ECallback2RT : protected EBaseCallback
{
public:
  ECallback2RT() {}

  /*! Tests if the callback has been set.
   * @return \c true if the callback has been set, otherwise \c false.
   */
  operator bool() const;

  /*! Calls the callback.
   */
  ReturnType operator ()(TP1 p1,TP2 p2) const;


  typedef ReturnType (*ECallerFn)(const EBaseCallback &cb,TP1 p1,TP2 p2);

  ECallback2RT( ECallerFn fnCaller,
             void *pObjReceiver,
             const void *fnMember,
             size_t nSizeFnMember );

private:
  ECallerFn _fnCaller;
};



template< class Function,class TP1,class TP2,class ReturnType >
class ECallbackCallerFn2RT : public ECallback2RT<TP1,TP2,ReturnType>
{
public:
  ECallbackCallerFn2RT( Function fnCallback );

  static ReturnType caller( const EBaseCallback &cb,TP1 p1,TP2 p2 );
};

/*! \brief This class provides a Prototype for static method callback.
 */
template<class ReturnType,class TP1,class TP2
         ,class RTpFn,class TP1Fn,class TP2Fn>
inline ECallbackCallerFn2RT< RTpFn (*)(TP1Fn,TP2Fn)
                                   ,TP1,TP2,ReturnType >
eCbCreator( ECallback2RT<TP1,TP2,ReturnType> *, 
                         RTpFn (*fn)(TP1Fn,TP2Fn) )
{
  return ECallbackCallerFn2RT< RTpFn (*)(TP1Fn,TP2Fn)
                            ,TP1,TP2,ReturnType>( fn );
}

template< class Receiver, class Member,class TP1,class TP2,class ReturnType>
class ECallbackCallerMember2RT : public ECallback2RT<TP1,TP2,ReturnType>
{
public:
  ECallbackCallerMember2RT( Receiver *pObjReceiver, 
                                  const Member &fnMember );


  static ReturnType caller( const EBaseCallback &cb,TP1 p1,TP2 p2 );
};


template< class Receiver, class CallObjType
,class RTpFn,class TP1Fn,class TP2Fn
,class TP1,class TP2,class ReturnType >
inline ECallbackCallerMember2RT< Receiver, 
                                       RTpFn (CallObjType::*)(TP1Fn,TP2Fn)
                                       ,TP1,TP2,ReturnType >
eCbCreator( ECallback2RT<TP1,TP2,ReturnType> *,
            Receiver &objReceiver,
            RTpFn (CallObjType::* const &fnMemberCallback)(TP1Fn,TP2Fn) )
{
  typedef RTpFn (CallObjType::*MemberCallback)(TP1Fn,TP2Fn);

  return ECallbackCallerMember2RT< Receiver, MemberCallback
                                ,TP1,TP2,ReturnType>( &objReceiver,
                                                            fnMemberCallback );
}






// ###################################################################
// Inline methods:
// ###################################################################


template<class TP1,class TP2,class ReturnType>
inline 
ECallback2RT<TP1,TP2,ReturnType>::operator bool() const
{
  return isValid();
}


template<class TP1,class TP2,class ReturnType>
inline 
ReturnType
ECallback2RT<TP1,TP2,ReturnType>::operator ()(TP1 p1,TP2 p2) const
{
  return _fnCaller( *this,p1,p2 );
}


template<class TP1,class TP2,class ReturnType>
inline
ECallback2RT<TP1,TP2,ReturnType>::ECallback2RT( ECallerFn fnCaller,
                      void *pObjReceiver,
                      const void *fnMember,
                      size_t nSizeFnMember ) :
    EBaseCallback( pObjReceiver,
                   fnMember,
                   nSizeFnMember ),
    _fnCaller( fnCaller )
{
}


template< class Function,class TP1,class TP2,class ReturnType >
inline
ECallbackCallerFn2RT< Function,TP1,TP2,ReturnType >::ECallbackCallerFn2RT( Function fnCallback ) :
    ECallback2RT<TP1,TP2,ReturnType>( caller, NULL, fnCallback, 0 )
{
}


template< class Function,class TP1,class TP2,class ReturnType >
inline ReturnType
ECallbackCallerFn2RT< Function,TP1,TP2,ReturnType >::caller( const EBaseCallback &cb,TP1 p1,TP2 p2 )
{
  return ((Function)cb._fnCallback)(p1,p2);
}


template< class Receiver, class Member,class TP1,class TP2,class ReturnType >
inline 
ECallbackCallerMember2RT<Receiver,Member,TP1,TP2,ReturnType>::ECallbackCallerMember2RT( Receiver *pObjReceiver, 
                                                               const Member &fnMember ) :
    ECallback2RT<TP1,TP2,ReturnType>( caller, pObjReceiver, &fnMember, sizeof( fnMember ) )
{
}

template< class Receiver, class Member,class TP1,class TP2,class ReturnType >
inline ReturnType
ECallbackCallerMember2RT<Receiver,Member,TP1,TP2,ReturnType>::caller( const EBaseCallback &cb,TP1 p1,TP2 p2 )
{
  Receiver &objReceiver = *(Receiver *)cb._pObjReceiver;
  Member &fnMember = *(Member *)(void *)cb._fnMemberCallback;

  return (objReceiver.*fnMember)(p1,p2);
}

#endif // !defined(AFX_ECALLBACK2RT_H__B78AA25B_D476_11D2_BD55_0040053AD648__INCLUDED_)
