// ###########################################################################
// File: ECallback3RT.h
// (c)Copyright 03/1999, Gaia Creation, Baptiste Lepilleur
// ###########################################################################
#if !defined(AFX_ECALLBACK3RT_H__B78AA25D_D476_11D2_BD55_0040053AD648__INCLUDED_)
#define AFX_ECALLBACK3RT_H__B78AA25D_D476_11D2_BD55_0040053AD648__INCLUDED_


#include "EBaseCallback.h"

/*! 
 * \brief This class provides a callback without parameters and return value.
 */
template<class TP1,class TP2,class TP3,class ReturnType>
class ECallback3RT : protected EBaseCallback
{
public:
  ECallback3RT() {}

  /*! Tests if the callback has been set.
   * @return \c true if the callback has been set, otherwise \c false.
   */
  operator bool() const;

  /*! Calls the callback.
   */
  ReturnType operator ()(TP1 p1,TP2 p2,TP3 p3) const;


  typedef ReturnType (*ECallerFn)(const EBaseCallback &cb,TP1 p1,TP2 p2,TP3 p3);

  ECallback3RT( ECallerFn fnCaller,
             void *pObjReceiver,
             const void *fnMember,
             size_t nSizeFnMember );

private:
  ECallerFn _fnCaller;
};



template< class Function,class TP1,class TP2,class TP3,class ReturnType >
class ECallbackCallerFn3RT : public ECallback3RT<TP1,TP2,TP3,ReturnType>
{
public:
  ECallbackCallerFn3RT( Function fnCallback );

  static ReturnType caller( const EBaseCallback &cb,TP1 p1,TP2 p2,TP3 p3 );
};

/*! \brief This class provides a Prototype for static method callback.
 */
template<class ReturnType,class TP1,class TP2,class TP3
         ,class RTpFn,class TP1Fn,class TP2Fn,class TP3Fn>
inline ECallbackCallerFn3RT< RTpFn (*)(TP1Fn,TP2Fn,TP3Fn)
                                   ,TP1,TP2,TP3,ReturnType >
eCbCreator( ECallback3RT<TP1,TP2,TP3,ReturnType> *, 
                         RTpFn (*fn)(TP1Fn,TP2Fn,TP3Fn) )
{
  return ECallbackCallerFn3RT< RTpFn (*)(TP1Fn,TP2Fn,TP3Fn)
                            ,TP1,TP2,TP3,ReturnType>( fn );
}

template< class Receiver, class Member,class TP1,class TP2,class TP3,class ReturnType>
class ECallbackCallerMember3RT : public ECallback3RT<TP1,TP2,TP3,ReturnType>
{
public:
  ECallbackCallerMember3RT( Receiver *pObjReceiver, 
                                  const Member &fnMember );


  static ReturnType caller( const EBaseCallback &cb,TP1 p1,TP2 p2,TP3 p3 );
};


template< class Receiver, class CallObjType
,class RTpFn,class TP1Fn,class TP2Fn,class TP3Fn
,class TP1,class TP2,class TP3,class ReturnType >
inline ECallbackCallerMember3RT< Receiver, 
                                       RTpFn (CallObjType::*)(TP1Fn,TP2Fn,TP3Fn)
                                       ,TP1,TP2,TP3,ReturnType >
eCbCreator( ECallback3RT<TP1,TP2,TP3,ReturnType> *,
            Receiver &objReceiver,
            RTpFn (CallObjType::* const &fnMemberCallback)(TP1Fn,TP2Fn,TP3Fn) )
{
  typedef RTpFn (CallObjType::*MemberCallback)(TP1Fn,TP2Fn,TP3Fn);

  return ECallbackCallerMember3RT< Receiver, MemberCallback
                                ,TP1,TP2,TP3,ReturnType>( &objReceiver,
                                                            fnMemberCallback );
}






// ###################################################################
// Inline methods:
// ###################################################################


template<class TP1,class TP2,class TP3,class ReturnType>
inline 
ECallback3RT<TP1,TP2,TP3,ReturnType>::operator bool() const
{
  return isValid();
}


template<class TP1,class TP2,class TP3,class ReturnType>
inline 
ReturnType
ECallback3RT<TP1,TP2,TP3,ReturnType>::operator ()(TP1 p1,TP2 p2,TP3 p3) const
{
  return _fnCaller( *this,p1,p2,p3 );
}


template<class TP1,class TP2,class TP3,class ReturnType>
inline
ECallback3RT<TP1,TP2,TP3,ReturnType>::ECallback3RT( ECallerFn fnCaller,
                      void *pObjReceiver,
                      const void *fnMember,
                      size_t nSizeFnMember ) :
    EBaseCallback( pObjReceiver,
                   fnMember,
                   nSizeFnMember ),
    _fnCaller( fnCaller )
{
}


template< class Function,class TP1,class TP2,class TP3,class ReturnType >
inline
ECallbackCallerFn3RT< Function,TP1,TP2,TP3,ReturnType >::ECallbackCallerFn3RT( Function fnCallback ) :
    ECallback3RT<TP1,TP2,TP3,ReturnType>( caller, NULL, fnCallback, 0 )
{
}


template< class Function,class TP1,class TP2,class TP3,class ReturnType >
inline ReturnType
ECallbackCallerFn3RT< Function,TP1,TP2,TP3,ReturnType >::caller( const EBaseCallback &cb,TP1 p1,TP2 p2,TP3 p3 )
{
  return ((Function)cb._fnCallback)(p1,p2,p3);
}


template< class Receiver, class Member,class TP1,class TP2,class TP3,class ReturnType >
inline 
ECallbackCallerMember3RT<Receiver,Member,TP1,TP2,TP3,ReturnType>::ECallbackCallerMember3RT( Receiver *pObjReceiver, 
                                                               const Member &fnMember ) :
    ECallback3RT<TP1,TP2,TP3,ReturnType>( caller, pObjReceiver, &fnMember, sizeof( fnMember ) )
{
}

template< class Receiver, class Member,class TP1,class TP2,class TP3,class ReturnType >
inline ReturnType
ECallbackCallerMember3RT<Receiver,Member,TP1,TP2,TP3,ReturnType>::caller( const EBaseCallback &cb,TP1 p1,TP2 p2,TP3 p3 )
{
  Receiver &objReceiver = *(Receiver *)cb._pObjReceiver;
  Member &fnMember = *(Member *)(void *)cb._fnMemberCallback;

  return (objReceiver.*fnMember)(p1,p2,p3);
}


#endif // !defined(AFX_ECALLBACK3RT_H__B78AA25D_D476_11D2_BD55_0040053AD648__INCLUDED_)
