// ###########################################################################
// File: ECallback5T.h
// (c)Copyright 03/1999, Gaia Creation, Baptiste Lepilleur
// ###########################################################################
#if !defined(AFX_ECALLBACK5T_H__B78AA261_D476_11D2_BD55_0040053AD648__INCLUDED_)
#define AFX_ECALLBACK5T_H__B78AA261_D476_11D2_BD55_0040053AD648__INCLUDED_


#include "EBaseCallback.h"

/*! 
 * \brief This class provides a callback without parameters and return value.
 */
template<class TP1,class TP2,class TP3,class TP4,class TP5>
class ECallback5T : protected EBaseCallback
{
public:
  ECallback5T() {}

  /*! Tests if the callback has been set.
   * @return \c true if the callback has been set, otherwise \c false.
   */
  operator bool() const;

  /*! Calls the callback.
   */
  void operator ()(TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5) const;


  typedef void (*ECallerFn)(const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5);

  ECallback5T( ECallerFn fnCaller,
             void *pObjReceiver,
             const void *fnMember,
             size_t nSizeFnMember );

private:
  ECallerFn _fnCaller;
};



template< class Function,class TP1,class TP2,class TP3,class TP4,class TP5 >
class ECallbackCallerFn5T : public ECallback5T<TP1,TP2,TP3,TP4,TP5>
{
public:
  ECallbackCallerFn5T( Function fnCallback );

  static void caller( const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5 );
};

/*! \brief This class provides a Prototype for static method callback.
 */
template<class ReturnType,class TP1,class TP2,class TP3,class TP4,class TP5
         ,class TP1Fn,class TP2Fn,class TP3Fn,class TP4Fn,class TP5Fn>
inline ECallbackCallerFn5T< ReturnType (*)(TP1Fn,TP2Fn,TP3Fn,TP4Fn,TP5Fn)
                                   ,TP1,TP2,TP3,TP4,TP5 >
eCbCreator( ECallback5T<TP1,TP2,TP3,TP4,TP5> *, 
                         ReturnType (*fn)(TP1Fn,TP2Fn,TP3Fn,TP4Fn,TP5Fn) )
{
  return ECallbackCallerFn5T< ReturnType (*)(TP1Fn,TP2Fn,TP3Fn,TP4Fn,TP5Fn)
                            ,TP1,TP2,TP3,TP4,TP5>( fn );
}

template< class Receiver, class Member,class TP1,class TP2,class TP3,class TP4,class TP5>
class ECallbackCallerMember5T : public ECallback5T<TP1,TP2,TP3,TP4,TP5>
{
public:
  ECallbackCallerMember5T( Receiver *pObjReceiver, 
                                  const Member &fnMember );


  static void caller( const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5 );
};


template< class Receiver, class CallObjType
,class TP1Fn,class TP2Fn,class TP3Fn,class TP4Fn,class TP5Fn
,class TP1,class TP2,class TP3,class TP4,class TP5,class ReturnType >
inline ECallbackCallerMember5T< Receiver, 
                                       ReturnType (CallObjType::*)(TP1Fn,TP2Fn,TP3Fn,TP4Fn,TP5Fn)
                                       ,TP1,TP2,TP3,TP4,TP5 >
eCbCreator( ECallback5T<TP1,TP2,TP3,TP4,TP5> *,
            Receiver &objReceiver,
            ReturnType (CallObjType::* const &fnMemberCallback)(TP1Fn,TP2Fn,TP3Fn,TP4Fn,TP5Fn) )
{
  typedef ReturnType (CallObjType::*MemberCallback)(TP1Fn,TP2Fn,TP3Fn,TP4Fn,TP5Fn);

  return ECallbackCallerMember5T< Receiver, MemberCallback
                                ,TP1,TP2,TP3,TP4,TP5>( &objReceiver,
                                                            fnMemberCallback );
}






// ###################################################################
// Inline methods:
// ###################################################################


template<class TP1,class TP2,class TP3,class TP4,class TP5>
inline 
ECallback5T<TP1,TP2,TP3,TP4,TP5>::operator bool() const
{
  return isValid();
}


template<class TP1,class TP2,class TP3,class TP4,class TP5>
inline 
void
ECallback5T<TP1,TP2,TP3,TP4,TP5>::operator ()(TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5) const
{
   _fnCaller( *this, p1,p2,p3,p4,p5 );
}


template<class TP1,class TP2,class TP3,class TP4,class TP5>
inline
ECallback5T<TP1,TP2,TP3,TP4,TP5>::ECallback5T( ECallerFn fnCaller,
                      void *pObjReceiver,
                      const void *fnMember,
                      size_t nSizeFnMember ) :
    EBaseCallback( pObjReceiver,
                   fnMember,
                   nSizeFnMember ),
    _fnCaller( fnCaller )
{
}


template< class Function,class TP1,class TP2,class TP3,class TP4,class TP5 >
inline
ECallbackCallerFn5T< Function,TP1,TP2,TP3,TP4,TP5 >::ECallbackCallerFn5T( Function fnCallback ) :
    ECallback5T<TP1,TP2,TP3,TP4,TP5>( caller, NULL, fnCallback, 0 )
{
}


template< class Function,class TP1,class TP2,class TP3,class TP4,class TP5 >
inline void
ECallbackCallerFn5T< Function,TP1,TP2,TP3,TP4,TP5 >::caller( const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5 )
{
   ((Function)cb._fnCallback)(p1,p2,p3,p4,p5);
}


template< class Receiver, class Member,class TP1,class TP2,class TP3,class TP4,class TP5 >
inline 
ECallbackCallerMember5T<Receiver,Member,TP1,TP2,TP3,TP4,TP5>::ECallbackCallerMember5T( Receiver *pObjReceiver, 
                                                               const Member &fnMember ) :
    ECallback5T<TP1,TP2,TP3,TP4,TP5>( caller, pObjReceiver, &fnMember, sizeof( fnMember ) )
{
}

template< class Receiver, class Member,class TP1,class TP2,class TP3,class TP4,class TP5 >
inline void
ECallbackCallerMember5T<Receiver,Member,TP1,TP2,TP3,TP4,TP5>::caller( const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5 )
{
  Receiver &objReceiver = *(Receiver *)cb._pObjReceiver;
  Member &fnMember = *(Member *)(void *)cb._fnMemberCallback;

   (objReceiver.*fnMember)(p1,p2,p3,p4,p5);
}

#endif // !defined(AFX_ECALLBACK5T_H__B78AA261_D476_11D2_BD55_0040053AD648__INCLUDED_)
