// ###########################################################################
// File: ECallback6T.h
// (c)Copyright 03/1999, Gaia Creation, Baptiste Lepilleur
// ###########################################################################
#if !defined(AFX_ECALLBACK6T_H__B78AA266_D476_11D2_BD55_0040053AD648__INCLUDED_)
#define AFX_ECALLBACK6T_H__B78AA266_D476_11D2_BD55_0040053AD648__INCLUDED_


#include "EBaseCallback.h"

/*! 
 * \brief This class provides a callback without parameters and return value.
 */
template<class TP1,class TP2,class TP3,class TP4,class TP5,class TP6>
class ECallback6T : protected EBaseCallback
{
public:
  ECallback6T() {}

  /*! Tests if the callback has been set.
   * @return \c true if the callback has been set, otherwise \c false.
   */
  operator bool() const;

  /*! Calls the callback.
   */
  void operator ()(TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5,TP6 p6) const;


  typedef void (*ECallerFn)(const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5,TP6 p6);

  ECallback6T( ECallerFn fnCaller,
             void *pObjReceiver,
             const void *fnMember,
             size_t nSizeFnMember );

private:
  ECallerFn _fnCaller;
};



template< class Function,class TP1,class TP2,class TP3,class TP4,class TP5,class TP6 >
class ECallbackCallerFn6T : public ECallback6T<TP1,TP2,TP3,TP4,TP5,TP6>
{
public:
  ECallbackCallerFn6T( Function fnCallback );

  static void caller( const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5,TP6 p6 );
};

/*! \brief This class provides a Prototype for static method callback.
 */
template<class ReturnType,class TP1,class TP2,class TP3,class TP4,class TP5,class TP6
         ,class TP1Fn,class TP2Fn,class TP3Fn,class TP4Fn,class TP5Fn,class TP6Fn>
inline ECallbackCallerFn6T< ReturnType (*)(TP1Fn,TP2Fn,TP3Fn,TP4Fn,TP5Fn,TP6Fn)
                                   ,TP1,TP2,TP3,TP4,TP5,TP6 >
eCbCreator( ECallback6T<TP1,TP2,TP3,TP4,TP5,TP6> *, 
                         ReturnType (*fn)(TP1Fn,TP2Fn,TP3Fn,TP4Fn,TP5Fn,TP6Fn) )
{
  return ECallbackCallerFn6T< ReturnType (*)(TP1Fn,TP2Fn,TP3Fn,TP4Fn,TP5Fn,TP6Fn)
                            ,TP1,TP2,TP3,TP4,TP5,TP6>( fn );
}

template< class Receiver, class Member,class TP1,class TP2,class TP3,class TP4,class TP5,class TP6>
class ECallbackCallerMember6T : public ECallback6T<TP1,TP2,TP3,TP4,TP5,TP6>
{
public:
  ECallbackCallerMember6T( Receiver *pObjReceiver, 
                                  const Member &fnMember );


  static void caller( const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5,TP6 p6 );
};


template< class Receiver, class CallObjType
,class TP1Fn,class TP2Fn,class TP3Fn,class TP4Fn,class TP5Fn,class TP6Fn
,class TP1,class TP2,class TP3,class TP4,class TP5,class TP6,class ReturnType >
inline ECallbackCallerMember6T< Receiver, 
                                       ReturnType (CallObjType::*)(TP1Fn,TP2Fn,TP3Fn,TP4Fn,TP5Fn,TP6Fn)
                                       ,TP1,TP2,TP3,TP4,TP5,TP6 >
eCbCreator( ECallback6T<TP1,TP2,TP3,TP4,TP5,TP6> *,
            Receiver &objReceiver,
            ReturnType (CallObjType::* const &fnMemberCallback)(TP1Fn,TP2Fn,TP3Fn,TP4Fn,TP5Fn,TP6Fn) )
{
  typedef ReturnType (CallObjType::*MemberCallback)(TP1Fn,TP2Fn,TP3Fn,TP4Fn,TP5Fn,TP6Fn);

  return ECallbackCallerMember6T< Receiver, MemberCallback
                                ,TP1,TP2,TP3,TP4,TP5,TP6>( &objReceiver,
                                                            fnMemberCallback );
}






// ###################################################################
// Inline methods:
// ###################################################################


template<class TP1,class TP2,class TP3,class TP4,class TP5,class TP6>
inline 
ECallback6T<TP1,TP2,TP3,TP4,TP5,TP6>::operator bool() const
{
  return isValid();
}


template<class TP1,class TP2,class TP3,class TP4,class TP5,class TP6>
inline 
void
ECallback6T<TP1,TP2,TP3,TP4,TP5,TP6>::operator ()(TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5,TP6 p6) const
{
   _fnCaller( *this, p1,p2,p3,p4,p5,p6 );
}


template<class TP1,class TP2,class TP3,class TP4,class TP5,class TP6>
inline
ECallback6T<TP1,TP2,TP3,TP4,TP5,TP6>::ECallback6T( ECallerFn fnCaller,
                      void *pObjReceiver,
                      const void *fnMember,
                      size_t nSizeFnMember ) :
    EBaseCallback( pObjReceiver,
                   fnMember,
                   nSizeFnMember ),
    _fnCaller( fnCaller )
{
}


template< class Function,class TP1,class TP2,class TP3,class TP4,class TP5,class TP6 >
inline
ECallbackCallerFn6T< Function,TP1,TP2,TP3,TP4,TP5,TP6 >::ECallbackCallerFn6T( Function fnCallback ) :
    ECallback6T<TP1,TP2,TP3,TP4,TP5,TP6>( caller, NULL, fnCallback, 0 )
{
}


template< class Function,class TP1,class TP2,class TP3,class TP4,class TP5,class TP6 >
inline void
ECallbackCallerFn6T< Function,TP1,TP2,TP3,TP4,TP5,TP6 >::caller( const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5,TP6 p6 )
{
   ((Function)cb._fnCallback)(p1,p2,p3,p4,p5,p6);
}


template< class Receiver, class Member,class TP1,class TP2,class TP3,class TP4,class TP5,class TP6 >
inline 
ECallbackCallerMember6T<Receiver,Member,TP1,TP2,TP3,TP4,TP5,TP6>::ECallbackCallerMember6T( Receiver *pObjReceiver, 
                                                               const Member &fnMember ) :
    ECallback6T<TP1,TP2,TP3,TP4,TP5,TP6>( caller, pObjReceiver, &fnMember, sizeof( fnMember ) )
{
}

template< class Receiver, class Member,class TP1,class TP2,class TP3,class TP4,class TP5,class TP6 >
inline void
ECallbackCallerMember6T<Receiver,Member,TP1,TP2,TP3,TP4,TP5,TP6>::caller( const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5,TP6 p6 )
{
  Receiver &objReceiver = *(Receiver *)cb._pObjReceiver;
  Member &fnMember = *(Member *)(void *)cb._fnMemberCallback;

   (objReceiver.*fnMember)(p1,p2,p3,p4,p5,p6);
}

#endif // !defined(AFX_ECALLBACK6T_H__B78AA266_D476_11D2_BD55_0040053AD648__INCLUDED_)
