// ###########################################################################
// File: ECallbackT.h
// (c)Copyright 03/1999, Gaia Creation, Baptiste Lepilleur
// ###########################################################################
#if !defined(AFX_ECALLBACKT_H__B78AA255_D476_11D2_BD55_0040053AD648__INCLUDED_)
#define AFX_ECALLBACKT_H__B78AA255_D476_11D2_BD55_0040053AD648__INCLUDED_


#include "EBaseCallback.h"

/*! 
 * \brief This class provides a callback without parameters and return value.
 */

class ECallbackT : protected EBaseCallback
{
public:
  ECallbackT() {}

  /*! Tests if the callback has been set.
   * @return \c true if the callback has been set, otherwise \c false.
   */
  operator bool() const;

  /*! Calls the callback.
   */
  void operator ()() const;


  typedef void (*ECallerFn)(const EBaseCallback &cb);

  ECallbackT( ECallerFn fnCaller,
             void *pObjReceiver,
             const void *fnMember,
             size_t nSizeFnMember );

private:
  ECallerFn _fnCaller;
};



template< class Function >
class ECallbackCallerFnT : public ECallbackT
{
public:
  ECallbackCallerFnT( Function fnCallback );

  static void caller( const EBaseCallback &cb );
};

/*! \brief This class provides a Prototype for static method callback.
 */
template<class ReturnType
         >
inline ECallbackCallerFnT< ReturnType (*)()
                                    >
eCbCreator( ECallbackT *, 
                         ReturnType (*fn)() )
{
  return ECallbackCallerFnT< ReturnType (*)()
                            >( fn );
}

template< class Receiver, class Member>
class ECallbackCallerMemberT : public ECallbackT
{
public:
  ECallbackCallerMemberT( Receiver *pObjReceiver, 
                                  const Member &fnMember );


  static void caller( const EBaseCallback &cb );
};


template< class Receiver, class CallObjType

,class ReturnType >
inline ECallbackCallerMemberT< Receiver, 
                                       ReturnType (CallObjType::*)()
                                        >
eCbCreator( ECallbackT *,
            Receiver &objReceiver,
            ReturnType (CallObjType::* const &fnMemberCallback)() )
{
  typedef ReturnType (CallObjType::*MemberCallback)();

  return ECallbackCallerMemberT< Receiver, MemberCallback
                                >( &objReceiver,
                                                            fnMemberCallback );
}






// ###################################################################
// Inline methods:
// ###################################################################



inline 
ECallbackT::operator bool() const
{
  return isValid();
}



inline 
void
ECallbackT::operator ()() const
{
   _fnCaller( *this );
}



inline
ECallbackT::ECallbackT( ECallerFn fnCaller,
                      void *pObjReceiver,
                      const void *fnMember,
                      size_t nSizeFnMember ) :
    EBaseCallback( pObjReceiver,
                   fnMember,
                   nSizeFnMember ),
    _fnCaller( fnCaller )
{
}


template< class Function >
inline
ECallbackCallerFnT< Function >::ECallbackCallerFnT( Function fnCallback ) :
    ECallbackT( caller, NULL, fnCallback, 0 )
{
}


template< class Function >
inline void
ECallbackCallerFnT< Function >::caller( const EBaseCallback &cb )
{
   ((Function)cb._fnCallback)();
}


template< class Receiver, class Member >
inline 
ECallbackCallerMemberT<Receiver,Member>::ECallbackCallerMemberT( Receiver *pObjReceiver, 
                                                               const Member &fnMember ) :
    ECallbackT( caller, pObjReceiver, &fnMember, sizeof( fnMember ) )
{
}

template< class Receiver, class Member >
inline void
ECallbackCallerMemberT<Receiver,Member>::caller( const EBaseCallback &cb )
{
  Receiver &objReceiver = *(Receiver *)cb._pObjReceiver;
  Member &fnMember = *(Member *)(void *)cb._fnMemberCallback;

   (objReceiver.*fnMember)();
}


#endif // !defined(AFX_ECALLBACKT_H__B78AA255_D476_11D2_BD55_0040053AD648__INCLUDED_)
