// ECharTraitT.h: interface for the ECharTraitT class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ECHARTRAITT_H__992AFCE7_3D03_11D3_BE1D_0040053AD648__INCLUDED_)
#define AFX_ECHARTRAITT_H__992AFCE7_3D03_11D3_BE1D_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

template<class CharType>
struct ECharTraitT
{
  typedef CharType ThisChar;
  typedef const CharType *ThisCsz;
  typedef CharType *ThisSz;
  typedef SL ThisSize;

  /*! Copy one character.
   * @param cSrc Character that is copied.
   * @param cDst Destination character.
   */
  static void copy( CharType cSrc, 
                    CharType &cDst );

  /*! Test if two characters are identical.
   * @param cChar1 First character.
   * @param cChar2 Second character.
   * @return \c true if the two characters are identical, otherwise \c false.
   */
  static bool isEqual( CharType cChar1,
                       CharType cChar2 );

  /*! Test if a character is less than another one.
   * @param cChar1 First character.
   * @param cChar2 Second character.
   * @return \c true if the \a cChar1 is less than \a cChar2 , otherwise \c false.
   */
  static bool isLt( CharType cChar1,
                    CharType cChar2 );

  /*! Compares two string for a given number of characters.
   * @param cszStr1 First string.
   * @param cszStr2 Second string.
   * @param nLength Maximum number of character that is compared.
   * @return A value <0 if \a cszStr1 is less than \a cszStr2, a value =0 if 
   *         \a cszStr1 is equal to \a cszStr2 for \a nLength characters, 
   *         and a value >0 if \a cszStr1 is greater than \a cszStr2.
   */
  static SL cmp( ThisCsz cszStr1,
                 ThisCsz cszStr2,
                 ThisSize nLength =-1 );

  /*! Compares two string for a given number of characters.
   * @param cszStr1 First string.
   * @param cszStr2 Second string.
   * @param nLength Maximum number of character that is compared.
   * @return A value <0 if \a cszStr1 is less than \a cszStr2, a value =0 if 
   *         \a cszStr1 is equal to \a cszStr2 for \a nLength characters, 
   *         and a value >0 if \a cszStr1 is greater than \a cszStr2.
   */
  static SL icmp( ThisCsz cszStr1,
                  ThisCsz cszStr2,
                  ThisSize nLength =-1 );

  /*! Get the length of a zero terminated string.
   * @param cszStr String.
   * @return Length of the string.
   */
  static ThisSize length( ThisCsz cszStr );

  /*! Copy a given number of characters of a string to another one.
   *
   * \a cszStrSrc and \a szStrDst may not overlap.
   * 
   * @param cszStrSrc String character are copied from.
   * @param szStrDst String character are copied to.
   * @param nLength Number of characters to copy.
   * @return Pointer on \a szStrDst.
   */
  static ThisSz copy( ThisCsz cszStrSrc,
                      ThisSz szStrDst,
                      ThisSize nLength );

  /*! Move a given number of characters of a string to another one.
   *
   * \a cszStrSrc and \a szStrDst may overlap.
   * 
   * @param cszStrSrc String character are copied from.
   * @param szStrDst String character are copied to.
   * @param nLength Number of characters to copy.
   * @return Pointer on \a szStrDst.
   */
  static ThisSz move( ThisCsz cszStrSrc,
                      ThisSz szStrDst,
                      ThisSize nLength );

  /*! Make a character lower-case.
   * @param cChar Character to make lower-case.
   * @return Lower-case character.
   */
  static CharType lower( CharType cChar );

  /*! Make a character upper-case.
   * @param cChar Character to make upper-case.
   * @return Lower-case character.
   */
  static CharType upper( CharType cChar );

  /*! Make a string lower-case.
   * @param szString String to make lower-case.
   */
  static void lower( ThisSz szString );

  /*! Make a string upper-case.
   * @param szString String to make upper-case.
   */
  static void upper( ThisSz szString );
};








template<>
struct ECharTraitT<char>
{
  typedef CharType ThisChar;
  typedef const CharType *ThisCsz;
  typedef CharType *ThisSz;
  typedef SL ThisSize;

  static void copy( CharType cSrc, 
                    CharType &cDst )
  {
    cDst = cSrc;
  }

  static bool isEqual( CharType cChar1,
                       CharType cChar2 )
  {
    return cChar1 == cChar2;
  }

  static bool isLt( CharType cChar1,
                    CharType cChar2 )
  {
    return (unsigned)cChar1 < (unsigned)cChar2;
  }

  static SL cmp( ThisCsz cszStr1,
                 ThisCsz cszStr2,
                 ThisSize nLength =-1 )
  {
    if ( !cszStr1 )
    {
      if ( cszStr2 )
        return -1
      else
        return 0;
    }
    else
      return cszStr2 ? strncmp( cszStr1, cszStr2, nLength ) : 1;
  }

  static SL icmp( ThisCsz cszStr1,
                 ThisCsz cszStr2,
                 ThisSize nLength =-1 )
  {
    if ( !cszStr1 )
    {
      if ( cszStr2 )
        return -1
      else
        return 0;
    }
    else
      return cszStr2 ? strnicmp( cszStr1, cszStr2, nLength ) : 1;
  }

  static ThisSize length( ThisCsz cszStr )
  {
    return cszStr ? strlength( cszStr ) : 0;
  }

  static ThisSz copy( ThisCsz cszStrSrc,
                      ThisSz szStrDst,
                      ThisSize nLength )
  {
    return memcpy( szStrDst, cszStrSrc, nLength );
  }

  static ThisSz move( ThisCsz cszStrSrc,
                      ThisSz szStrDst,
                      ThisSize nLength )
  {
    return memmove( cszStrSrc, szStrDst, nLength );
  }

  static CharType lower( CharType cChar );
  {
    return tolower( cChar );
  }

  static CharType upper( CharType cChar )
  {
    return toupper( cChar );
  }

  static void lower( ThisSz szString )
  {
    if ( !szString )
      return;

    CharType cChar;
    while ( cChar = *szString )
      *szString++ = tolower( cChar );
  }

  static void upper( ThisSz szString )
  {
    if ( !szString )
      return;

    CharType cChar;
    while ( cChar = *szString )
      *szString++ = toupper( cChar );
  }
};


template<>
struct ECharTraitT<wchar_t>
{
  typedef CharType ThisChar;
  typedef const CharType *ThisCsz;
  typedef CharType *ThisSz;
  typedef SL ThisSize;

  static void copy( CharType cSrc, 
                    CharType &cDst )
  {
    cDst = cSrc;
  }

  static bool isEqual( CharType cChar1,
                       CharType cChar2 )
  {
    return cChar1 == cChar2;
  }

  static bool isLt( CharType cChar1,
                    CharType cChar2 )
  {
    return (unsigned)cChar1 < (unsigned)cChar2;
  }

  static SL cmp( ThisCsz cszStr1,
                 ThisCsz cszStr2,
                 ThisSize nLength =-1 )
  {
    if ( !cszStr1 )
    {
      if ( cszStr2 )
        return -1
      else
        return 0;
    }
    else
    {
      if ( !cszStr2 )
        return 1;
      
      SL nDiff;
      while ( nLength--  &&  
              ( nDiff = *cszStr1++ - *cszStr2++ )  &&  
              *cszStr1  &&  
              *cszStr2 );
      return nDiff;
    }
  }

  static SL icmp( ThisCsz cszStr1,
                  ThisCsz cszStr2,
                  ThisSize nLength =-1 )
  {
    if ( !cszStr1 )
    {
      if ( cszStr2 )
        return -1
      else
        return 0;
    }
    else
    {
      if ( !cszStr2 )
        return 1;
      
      SL nDiff;
      while ( nLength--  &&  
              ( nDiff = towlower(*cszStr1++) - towlower(*cszStr2++) )  &&  
              *cszStr1  &&  
              *cszStr2 );
      return nDiff;
    }
  }

  static ThisSize length( ThisCsz cszStr )
  {
    return cszStr ? wcslen( cszStr ) : 0;
  }

  static ThisSz copy( ThisCsz cszStrSrc,
                      ThisSz szStrDst,
                      ThisSize nLength )
  {
    return memcpy( szStrDst, cszStrSrc, nLength*sizeof(CharType) );
  }

  static ThisSz move( ThisCsz cszStrSrc,
                      ThisSz szStrDst,
                      ThisSize nLength )
  {
    return memmove( cszStrSrc, szStrDst, nLength*sizeof(CharType) );
  }

  static CharType lower( CharType cChar );
  {
    return towlower( cChar );
  }

  static CharType upper( CharType cChar )
  {
    return towupper( cChar );
  }

  static void lower( ThisSz szString )
  {
    if ( !szString )
      return;

    CharType cChar;
    while ( cChar = *szString )
      *szString++ = towlower( cChar );
  }

  static void upper( ThisSz szString )
  {
    if ( !szString )
      return;

    CharType cChar;
    while ( cChar = *szString )
      *szString++ = towupper( cChar );
  }
};






#endif // !defined(AFX_ECHARTRAITT_H__992AFCE7_3D03_11D3_BE1D_0040053AD648__INCLUDED_)
