// EClassId.h: interface for the EClassId class.
//
//////////////////////////////////////////////////////////////////////

/* HST:
 * 18/10/1998: - BugFix: Bad assertion in EClassId::createObject().
 * 03/03/1999: - add EDECL_COPY & EIMPL_COPY.
 */


#if !defined(AFX_ECLASSID_H__25480D13_4886_11D2_AE44_E6FF6EE0B361__INCLUDED_)
#define AFX_ECLASSID_H__25480D13_4886_11D2_AE44_E6FF6EE0B361__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "EString.hpp"
#include "EBaseClass.h"
#include "EObjectContext.h"

#define ECLASSID_NAME( ClassName ) ClassName::_classId

#define EDECL_CLASSID                                              \
  static const EClassId _classId;                                  \
  static const EClassId &classId() { return _classId; }            \
  virtual const EClassId &objectId() const { return classId(); }
    

#define EIMPL_CLASSID( ClassName )                             \
  const EClassId ECLASSID_NAME(ClassName)( #ClassName,         \
                                           NULL );                       

#define EDECL_DYNCLASS                                              \
  static const EClassId _classId;                                   \
  static EBaseClass *createNewObject();                             \
  static const EClassId &classId() { return _classId; }             \
  virtual const EClassId &objectId() const { return classId(); }
    

#define EIMPL_DYNCLASS( ClassName )                             \
  const EClassId ECLASSID_NAME(ClassName)( #ClassName,                   \
                                           ClassName::createNewObject ); \
  EBaseClass *ClassName::createNewObject()                      \
  {                                                             \
    return new ClassName;                                       \
  } 

#define EDECL_PERSCLASS EDECL_DYNCLASS

#define EIMPL_PERSCLASS( ClassName, Version)                             \
  const EClassId ECLASSID_NAME(ClassName)( #ClassName,                   \
                                           ClassName::createNewObject,   \
                                           Version );           \
  EBaseClass *ClassName::createNewObject()                      \
  {                                                             \
    return new ClassName;                                       \
  } 


#define ECLASSID( ClassName )   (ClassName::classId())

#define EOBJECTID( Object ) ((Object).objectId())

#ifdef NDEBUG
#  define ESAMECLASS( objectA, objectB )
#  define EINHERIT( objectA, clsid )
#else
#  define ESAMECLASS( objectA, objectB ) \
   EASSERT( &EOBJECTID(objectA) == &EOBJECTID(objectB) );
#  define EINHERIT( objectA, clsid ) 
#endif

/*! Defines a copy() method and a duplicate() method.
 *
 *  Implementation of those methods require that the object
 * has a copy constructor and a copy operator.
 */
#define EDECL_COPY                                \
  virtual void copy( EBaseClass &objDest ) const; \
  virtual EBaseClass *duplicate() const;          

/*! Implements a copy() and a duplicate() method.
 */
#define EIMPL_COPY( ClassName )                     \
  void ClassName::copy( EBaseClass &objDest ) const \
  {                                                 \
    ESAMECLASS( *this, objDest );                   \
    ((ClassName &)objDest) = *this;                 \
  }                                                 \
  EBaseClass *ClassName::duplicate() const          \
  {                                                 \
    return new ClassName( *this );                  \
  }                                                 

/*! Defines a copy() method and a duplicate() and a copy constructor method.
 *
 *  Implementation of those methods require that the object
 * has a copy constructor and a copy operator.
 */
#define EDECL_COPYCTOR( ClassName )               \
  virtual void copy( EBaseClass &objDest ) const; \
  virtual EBaseClass *duplicate() const;          \
  ClassName( const ClassName &src );              \
  ClassName &operator =( const ClassName &src );   


/*! Implements a copy() and a duplicate() and a copy constructor method.
 */
#define EIMPL_COPYCTOR( ClassName )                  \
  void ClassName::copy( EBaseClass &objDest ) const  \
  {                                                  \
    ESAMECLASS( *this, objDest );                    \
    ((ClassName &)objDest) = *this;                  \
  }                                                  \
  EBaseClass *ClassName::duplicate() const           \
  {                                                  \
    return new ClassName( *this );                   \
  }                                                  \
  ClassName::ClassName( const ClassName &src )       \
  {                                                  \
    *this = src;                                     \
  }                                                 

/*! This class provides class identifiaction, name and dynamic creation.
 *
 * Example of use:
 * \code
 * // header
 * class MyClass : public EBaseClass
 * {
 * public:
 *   EDECL_DYNCLASS( MyClass );
 *
 *   // Dynamic class must have a default constructor.
 *   MyClass() {}
 * };
 *
 * // source
 * EIMPL_DYNCLASS( MyClass );
 *
 * // using dynamic creation:
 * // The EClassId object associated to the class can be retreive using the
 * // macro ECLASSID( ClassName ). The objet can be used to retreive class name
 * // or to create a new object of this class.
 *
 * EBaseClass *pObject = ECLASSID(MyClass).createObject();
 *
 * // You can later retreive the class id of any object :
 * EOBJECTID( *pObjet ).className();    // will return "MyClass".
 * \endcode
 */
class EClassId  
{
public:
	/*! Constructs a new class id object.
   * @param strClassName Name of the class.
   * @param pfnCreateNewObject Function used to create a new object. 
   *        pfnCreateNewObject can be NULL if the class does not support 
   *        dynamic creation.
   */
  EClassId( const EString &strClassName,
            EBaseClass *(*pfnCreateNewObject)() );

  /*! Constructs a new class id object.
   * @param strClassName Name of the class.
   * @param pfnCreateNewObject Function used to create a new object. 
   *        pfnCreateNewObject can be NULL if the class does not support 
   *        dynamic creation.
   * @parma nVersion Storage format version.
   */
  EClassId( const EString &strClassName,
            EBaseClass *(*pfnCreateNewObject)(),
            UL nVersion );

  /*! Destructor.
   */
  ~EClassId() {};

  /*! Get the class name;
   * @return Class name.
   */
  const EString &className() const;

  /*! Create a new object of the class associated to this object.
   * @return New object.
   */
  EBaseClass *createObject() const;

  /*! Get the first class id in the list.
   * @return First class id.
   */
  static const EClassId *firstClass();

  /*! Get the next class id in the list.
   * @return Class Id following this one. (NULL if none).
   */
  const EClassId *nextClass() const;

  /*! Find the class id of a class giving its name.
   * @param strClassName Name of the class to find.
   * @return Class id of the class. \c NULL if not found.
   */
  static const EClassId *findClass( const EString &strClassName );

  /*! Get class storage format version number.
   * @return Class storage format version number.
   */
  UL version() const;

private:
  /*! Class name.
   */
  EString _strClassName;

  /*! Function used to create an object based on this class.
   */
  EBaseClass *(*_pfnCreateNewObject)();

  /*! Linked list of registred class id.
   */
  EClassId *_pNextClassId;

  /*! Head of the linked list of registred class id.
   */
  static EClassId *_pFirstClassId;

  /*! Class version, used for persistance storage.
   */
  UL _nVersion;
};




inline const EString &
EClassId::className() const
{
  return _strClassName;
}


inline EBaseClass *
EClassId::createObject() const
{
  EASSERT( _pfnCreateNewObject );
  return _pfnCreateNewObject();
}


inline const EClassId *
EClassId::firstClass()
{
  return _pFirstClassId;
}


inline const EClassId *
EClassId::nextClass() const
{
  return _pNextClassId;
}


inline UL 
EClassId::version() const
{
  return _nVersion;
}


#endif // !defined(AFX_ECLASSID_H__25480D13_4886_11D2_AE44_E6FF6EE0B361__INCLUDED_)
