// EDirectory.cpp: implementation of the EDirectory class.
//
//////////////////////////////////////////////////////////////////////

#include "usuel.hpp"
#include "EAllocators.h"
#include "EArray.h"
#include "EDirectory.h"
#include "EMath.hpp"

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN 
#define NOGDI
#define NOUSER
#define NONLS
#define NOMINMAX

#include "windows.h"
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

EDirectory::EDirectory()
{

}

EDirectory::~EDirectory()
{

}


EDirectory::EDirectory( const EString &strName )
{
  setName( strName );
}


void 
EDirectory::setName( const EString &strName )
{
  if ( strName.lastChar() == '\\'  ||  strName.lastChar() == '/' )
    _strName = strName;
  else
    _strName = strName + '\\';
}


const 
EString &
EDirectory::name() const
{
  return _strName;
}


void 
EDirectory::findFile( EFilenameList &listFilename,
                      const EString &strFilter ) const
{
#ifndef WIN32
  EASSERT( false );
#else
  // Win32 FindFile version


  EString strFilename = name() + strFilter;
  WIN32_FIND_DATA findData;
  HANDLE hFind = FindFirstFile( (LPCTSTR)strFilename.cstr(),
                                &findData );
  if ( hFind == INVALID_HANDLE_VALUE )
    return;

  EFilename filename;

  bool bContinue;
  do
  {
    strFilename = name() + findData.cFileName;
    if ( (findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0 )
      strFilename += '\\';

    filename.setFilename( strFilename );
    listFilename.append( filename );
    bContinue = FindNextFile( hFind,
                              &findData ) ? true : false;
  }
  while ( bContinue );

  FindClose( hFind );

#endif
}


void 
EDirectory::setToParent()
{
  if ( _strName.length() )
  {
    TCHAR c = _strName.lastChar();
    SL nStart = ( c == '\\'  ||  c == '/' ) ? -2 : -1;
    SL nPosSlash = emax( _strName.posLast( '\\', nStart ),
                         _strName.posLast( '/', nStart ) );
    if ( nPosSlash >= 0 )
    {
      _strName.resize( nPosSlash +1 );
    }
  }
}
