// EHashString.h: interface for the EHashString class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EHASHSTRING_H__FEA0CFAC_91FD_11D2_AF1C_AED26EBCA961__INCLUDED_)
#define AFX_EHASHSTRING_H__FEA0CFAC_91FD_11D2_AF1C_AED26EBCA961__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "EString.hpp"

/*! \brief This class provides a hash-table with string key.
 * 
 *  Comparison are case sensitive. Use EHashStringI for insensitive comparison.
 *
 * @param Item Item.
 * @param ItemArg Item when passed has argument.
 */
template<class Item, class ItemArg>
class EHashString : public EHash<EString,Item,const EString &,ItemArg>
{
public:
  typedef EString ThisString;

  /*! Constructs the hash-table.
   * @param nSize initial size of the table.
   */
  EHashString( SL nSize = 3 ) : EHash<EString,Item,const EString &,ItemArg>( nSize ) {}

  /*! Destructor.
   */
  virtual ~EHashString() {}

  /*! Build a numeric value from the key.
   * @param key key from which the value is build.
   * @return numeric value.
   */
  virtual UL getKey( const EString &key ) const;

  /*! Test if two key are equal.
   * @param key1 key to compare to key2.
   * @param key2 key to compare to key1.
   * @return \c true if the key are equal, \c false otherwise.
   */
  virtual bool compareKey( const EString &key1, 
                           const EString &key2 ) const;

};


template<class Item, class ItemArg>
UL 
EHashString<Item,ItemArg>::getKey( const EString &key ) const
{
  return key.makeHashKey();
}


template<class Item, class ItemArg>
bool 
EHashString<Item,ItemArg>::compareKey( const EString &key1, 
                                       const EString &key2 ) const
{
  return key1 == key2;
}


#endif // !defined(AFX_EHASHSTRING_H__FEA0CFAC_91FD_11D2_AF1C_AED26EBCA961__INCLUDED_)
