// EHashStringI.h: interface for the EHashStringI class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EHASHSTRINGI_H__FEA0CFAD_91FD_11D2_AF1C_AED26EBCA961__INCLUDED_)
#define AFX_EHASHSTRINGI_H__FEA0CFAD_91FD_11D2_AF1C_AED26EBCA961__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "EString.hpp"

/*! \brief This class provides a hash-table with string key.
 * 
 *  Comparison are case insensitive. Use EHashString for sensitive comparison.
 *
 * @param Item Item.
 * @param ItemArg Item when passed has argument.
 */
template<class Item, class ItemArg>
class EHashStringI : public EHash<EString,Item,const EString &,ItemArg>
{
public:
  typedef EString ThisString;

  /*! Constructs the hash-table.
   * @param nSize initial size of the table.
   */
  EHashStringI( SL nSize = 3 ) : EHash<EString,Item,const EString &,ItemArg>( nSize ) {}

  /*! Destructor.
   */
  virtual ~EHashStringI() {}

  /*! Build a numeric value from the key.
   * @param key key from which the value is build.
   * @return numeric value.
   */
  virtual UL getKey( const EString &key ) const;

  /*! Test if two key are equal.
   * @param key1 key to compare to key2.
   * @param key2 key to compare to key1.
   * @return \c true if the key are equal, \c false otherwise.
   */
  virtual bool compareKey( const EString &key1, 
                           const EString &key2 ) const;

};


template<class Item, class ItemArg>
UL 
EHashStringI<Item,ItemArg>::getKey( const EString &key ) const
{
  return key.imakeHashKey();
}


template<class Item, class ItemArg>
bool 
EHashStringI<Item,ItemArg>::compareKey( const EString &key1, 
                                        const EString &key2 ) const
{
  return key1.icmp( key2 ) == 0;
}

#endif // !defined(AFX_EHASHSTRINGI_H__FEA0CFAD_91FD_11D2_AF1C_AED26EBCA961__INCLUDED_)
