// EListenerList.h: interface for the EListenerList class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ELISTENERLIST_H__BFF2A860_5DD2_4C7F_82A8_238FB8FC8C1B__INCLUDED_)
#define AFX_ELISTENERLIST_H__BFF2A860_5DD2_4C7F_82A8_238FB8FC8C1B__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "EArray.h"
#include "EArrayIterator.h"


/*! \brief This class represents a list of Listener.
 * \include "EListenerList.h"
 *
 * The listeners are not owned by the list and therefore it is not its responsability
 * to destroy them.
 *
 * @param Listener Type of listener (not a pointer).
 */
template<class Listener>
class EListenerList
{
public:
  typedef Listener TypeListener;
  typedef EListenerList<Listener> TypeListListener;
  typedef EBinDynArrayT<TypeListener *> ArrayListener;
  typedef typename ArrayListener::EIterator ItListener;

  /*!
   * Constructs a list of listener.
   */
  EListenerList() {}

  /*!
   * Destructor.
   */
  virtual ~EListenerList() {}

  /*!
   * Adds a listener to the list.
   * @param pListener Pointer on the listener to add. Not added if \c NULL.
   */
  virtual void addListener( TypeListener *pListener );

  /*!
   * Removes a listener from the list.
   * @param pListener Pointer on the listener to remove.
   */
  virtual void removeListener( TypeListener *pListener );

  /*!
   * Iterates the list.
   * @return An iterator on the list of listener.
   */
  virtual ItListener iterator() const;

private:
  typedef EBinDynArrayT<TypeListener *> ArrayListener;
  ArrayListener _apListener;
};


// Implementation of EListenerList:
// --------------------------------

template<class Listener>
void 
EListenerList<Listener>::addListener( TypeListener *pListener )
{
  if ( pListener )
    _apListener.append( pListener );
}


template<class Listener>
void 
EListenerList<Listener>::removeListener( TypeListener *pListener )
{
  _apListener.remove( pListener );
}


template<class Listener>
typename EListenerList<Listener>::ItListener 
EListenerList<Listener>::iterator() const
{
  return ItListener( _apListener );
}


#endif // !defined(AFX_ELISTENERLIST_H__BFF2A860_5DD2_4C7F_82A8_238FB8FC8C1B__INCLUDED_)
