// ESimpleAllocator.h: interface for the ESimpleAllocator class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ESIMPLEALLOCATOR_H__FD31CFCA_BB54_11D2_91EF_004005446EE6__INCLUDED_)
#define AFX_ESIMPLEALLOCATOR_H__FD31CFCA_BB54_11D2_91EF_004005446EE6__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "EAllocatorBin.h"
#include "EAllocator.h"

template< class TElm, class EParentAllocator=EAllocatorBinT<TElm> >
class ESimpleAllocatorT : public EParentAllocator
{
public:
  /*! Constructs the allocator.
   * 
   * Set the array to NULL.
   */
	ESimpleAllocatorT();

  /*! Destructs the allocator.
   *
   * \warning Child class must do the own to resize(0) if the destruct() method
   * is overloaded because the overrided destruct() method will not be called at
   * this level.
   */
	virtual ~ESimpleAllocatorT();

  /*! Get a reference on a object of a given index.
   * @param nIndex Zero based index of the object.
   * @return Reference on the object.
   */
  virtual const TElm &at( SL nIndex ) const;

  /*! Get a reference on a object of a given index.
   * @param nIndex Zero based index of the object.
   * @return Reference on the object.
   */
  virtual TElm &at( SL nIndex );

  /*! Resizes the array so that a given number of elements fit in.
   * @param nNewSize Number of elements that should fit in the array.
   *                 If 0 then the array is freeed.
   */
  virtual void resize( SL nNewSize );

  /*! Returns the exact number of elements in the array.
   * @return Number of elements in the array.
   */
  virtual SL count() const;

private:
  /**@shapeType DependencyLink*/
  /*#  EAllocatorT lnkUnnamed*/
  TElm *_pDatas;
  SL _nCount;
};



EDECLARE_ALLOCATOR( ESimpleAllocatorT, EAllocatorBinT, ESimpleAllocatorBinT );
EDECLARE_ALLOCATOR( ESimpleAllocatorT, EAllocatorObjT, ESimpleAllocatorObjT );
EDECLARE_ALLOCATOR( ESimpleAllocatorT, EAllocatorPtrT, ESimpleAllocatorPtrT );



template<class TElm,class EParentAllocator>
ESimpleAllocatorT<TElm,EParentAllocator>::ESimpleAllocatorT()
{
  _pDatas = NULL;
  _nCount = 0;
}


template<class TElm,class EParentAllocator>
ESimpleAllocatorT<TElm,EParentAllocator>::~ESimpleAllocatorT()
{
  resize( 0 );
}


template<class TElm,class EParentAllocator>
void 
ESimpleAllocatorT<TElm,EParentAllocator>::resize( SL nNewSize )
{
  EASSERT( nNewSize >= 0 );

  SL nCurSize = count();

  if ( nNewSize == nCurSize )   // no change 
    return;
  
  if ( nNewSize < nCurSize )
    destruct( nNewSize, nCurSize - nNewSize );

  if ( !nNewSize )      // free datas array
  {
    if ( _pDatas )
    {
      efree( _pDatas );
      _nCount = 0;
      _pDatas = ENIL;
    }
  }
  else                  // resize array
  {
    _pDatas = (TElm *)erealloc( _pDatas, 
                                sizeof( TElm ) * nNewSize );
    _nCount = nNewSize;

    if ( nCurSize < nNewSize )
    {
      construct( nCurSize,
                 nNewSize - nCurSize );
    }
  }
}


template<class TElm,class EParentAllocator>
inline SL 
ESimpleAllocatorT<TElm,EParentAllocator>::count() const
{
  return _nCount;
}


template<class TElm,class EParentAllocator>
const TElm &
ESimpleAllocatorT<TElm,EParentAllocator>::at( SL nIndex ) const
{
  EASSERT( nIndex >= 0 );
  EASSERT( nIndex < _nCount );
  return _pDatas[ nIndex ];
}


template<class TElm,class EParentAllocator>
TElm &
ESimpleAllocatorT<TElm,EParentAllocator>::at( SL nIndex )
{
  EASSERT( nIndex >= 0 );
  EASSERT( nIndex < _nCount );
  return _pDatas[ nIndex ];
}

#endif // !defined(AFX_ESIMPLEALLOCATOR_H__FD31CFCA_BB54_11D2_91EF_004005446EE6__INCLUDED_)
