// ESortedCollectionT.h: interface for the ESortedCollectionT class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ESORTEDCOLLECTIONT_H__32671F93_3F99_11D3_BE24_0040053AD648__INCLUDED_)
#define AFX_ESORTEDCOLLECTIONT_H__32671F93_3F99_11D3_BE24_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

template<class Arg1, class Arg2, class Result>
struct EFn2T
{
  typedef Arg1 ThisArg1;
  typedef Arg2 ThisArg2;
  typedef Result ThisResult;
};

template<class Arg>
struct EFn2LessT : public EFn2T<Arg,Arg,bool>
{
  bool operator ()( const Arg &arg1,
                    const Arg &arg2 ) const;
};

template<class Arg>
struct EFn2LessOpT : public EFn2T<Arg,Arg,bool>
{
  bool operator ()( const Arg &arg1,
                    const Arg &arg2 ) const
  {
    return arg1 < arg2;
  }
};

template<class Arg>
struct EFn2GreaterT : public EFn2T<Arg,Arg,bool>
{
  bool operator ()( const Arg &arg1,
                    const Arg &arg2 ) const;
};

template<class Arg>
struct EFn2GreaterOpT : public EFn2T<Arg,Arg,bool>
{
  bool operator ()( const Arg &arg1,
                    const Arg &arg2 ) const
  {
    return arg1 > arg2;
  }
};

template<class Arg>
struct EFn2EqualT : public EFn2T<Arg,Arg,bool>
{
  bool operator ()( const Arg &arg1,
                    const Arg &arg2 ) const;
};

template<class Arg>
struct EFn2EqualOpT : public EFn2T<Arg,Arg,bool>
{
  bool operator ()( const Arg &arg1,
                    const Arg &arg2 ) const
  {
    return arg1 == arg2;
  }
};

template<class Item,class Iterator,class Pred>
SL
ebinarySearchLowerBound( Iterator &it,
                         const Item &item,
                         Pred &predicate )
{
  SL nCount = it.count();
  SL nMin =0;
  
  while ( 0 < nCount )
  {
    SL nCount2 = nCount /2;
    SL nMid = nMin + nCount2;
    if ( predicate( item, it.at( nMid ) ) )
    {
      nCount = nMid;
    }
    else
    {
      nMin = nMid+1;
      nCount -= nCount2 +1;
    }
  }

  it.seekTo( nMin );
  return nMin;
}


template<class Item,class Iterator,class Pred>
SL
ebinarySearchUpperBound( Iterator &it,
                         const Item &item,
                         Pred &predicate )
{
  SL nCount = it.count();
  SL nMin =0;
  
  while ( 0 < nCount )
  {
    SL nCount2 = nCount /2;
    SL nMid = nMin + nCount2;
    if ( predicate( it.at( nMid ), item ) )
    {
      nCount = nMid;
    }
    else
    {
      nMin = nMid+1;
      nCount -= nCount2 +1;
    }
  }

  it.seekTo( nMin );
  return nMin;
}




/*! \class ESortedCollectionT<Item,Collection>
 * \include "ESortedCollectionT.h"
 * \brief Template that provide sorted collection management.
 *
 * The class \a Collection must have a EIterator of type ERandomItT.
 */
template< class Item, 
          class Collection, 
          class FnLess =EFn2LessOpT<Item>,
          class FnEqual =EFn2EqualOpT<Item> >
class ESortedCollectionT
{
public:
  typedef Item ThisItem;
  typedef Collection ThisCollection;
  typedef typename Collection::EIterator ThisIterator;
  typedef FnLess ThisFnLess;
  typedef FnEqual ThisFnEqual;
  typedef ESortedCollectionT<Item,Collection,FnLess,FnEqual> ThisSortedCollection;

  /*! Default constructor.
   *
   * No collection is attached.
   */
  ESortedCollectionT() {}

  /*! Destructor.
   */
  virtual ~ESortedCollectionT() {}

  /*! Adds an item to the collection.
   * @param item Item to add.
   * @return Position of the item was inserted.
   */
  SL add( const ThisItem &item );

  /*! Removes an item from the collection.
   * @param item Item to remove.
   * @return Old position of the item.
   */
  SL remove( const ThisItem &item );

  /*! Finds the position of an item.
   * 
   * @param item Item to find.
   * @return Position of the item. -1 if the item does not exist.
   */
  SL find( const ThisItem &item ) const;

  /*! Finds the position of the previous item.
   * @return Position of the last item for which the predicate
   * \c FnLess( item, itemCompared ) is \c false.
   */
  SL findLower( const ThisItem &item ) const;

  /*! Finds the position of the greater item.
   * 
   * @param item Item used for the search.
   * @return Position of the first item that is greater than \a item.
   */
  SL findGreater( const ThisItem &item ) const;

  /*! Tests if an item is in the collection.
   * @param item Item which existence is tested.
   * @return \c true if the item is in the collection, otherwise \c false.
   */
  bool isIn( const ThisItem &item ) const;

  /*! Get the collection.
   * @return The collection.
   */
  Collection &collection();

  /*! Get the collection.
   * @return The collection.
   */
  const Collection &collection() const;

  /*! Adds the item of an unsorted collection to this one.
   */
  void add( const Collection &collection );

  void merge( const ThisSortedCollection &sorted );

protected:
  FnLess _fnLess;
  FnEqual _fnEqual;

private:
  /*! The collection.
   */
  Collection _collection;
};


template< class Item, 
          class Collection, 
          class FnLess,
          class FnEqual >
inline Collection &
ESortedCollectionT<Item,Collection,FnLess,FnEqual>::collection()
{
  return _collection;
}


template< class Item, 
          class Collection, 
          class FnLess,
          class FnEqual >
inline const Collection &
ESortedCollectionT<Item,Collection,FnLess,FnEqual>::collection() const
{
  return _collection;
}


template< class Item, 
          class Collection, 
          class FnLess,
          class FnEqual >
inline SL 
ESortedCollectionT<Item,Collection,FnLess,FnEqual>::add( const ThisItem &item )
{
  EASSERT( !isIn( item ) );
  SL nPos = findLower( item );
  collection().insert( nPos,
                       item );
  return nPos;
}


template< class Item, 
          class Collection, 
          class FnLess,
          class FnEqual >
inline SL 
ESortedCollectionT<Item,Collection,FnLess,FnEqual>::remove( const ThisItem &item )
{
  SL nPos = find( item );
  if ( nPos >= 0 )
  {
    collection().remove( nPos );
  }
  return nPos;
}


template< class Item, 
          class Collection, 
          class FnLess,
          class FnEqual >
SL 
ESortedCollectionT<Item,Collection,FnLess,FnEqual>::find( const ThisItem &item ) const
{
  ThisIterator it( collection() );
  SL nPos = ebinarySearchLowerBound( it, item, ThisFnLess() );
  if ( _fnEqual(it.at( nPos ),item ) )
    return nPos;
  return -1;
}


template< class Item, 
          class Collection, 
          class FnLess,
          class FnEqual >
SL 
ESortedCollectionT<Item,Collection,FnLess,FnEqual>::findLower( const ThisItem &item ) const
{
  ThisIterator it( collection() );
  return ebinarySearchLowerBound( it, item, ThisFnLess() );
}


template< class Item, 
          class Collection, 
          class FnLess,
          class FnEqual >
inline SL 
ESortedCollectionT<Item,Collection,FnLess,FnEqual>::findGreater( const ThisItem &item ) const
{
  ThisIterator it( collection() );
  return ebinarySearchUpperBound( it, item, ThisFnLess() );
}


template< class Item, 
          class Collection, 
          class FnLess,
          class FnEqual >
inline bool 
ESortedCollectionT<Item,Collection,FnLess,FnEqual>::isIn( const ThisItem &item ) const
{
  return find( item ) != -1;
}


template< class Item, 
          class Collection, 
          class FnLess,
          class FnEqual >
void 
ESortedCollectionT<Item,Collection,FnLess,FnEqual>::merge( const ThisSortedCollection &sorted )
{
  ThisIterator itThis( collection() );
  ThisIterator it( sorted.collection() );

  itThis.begin();
  it.begin();

  while ( !it.isEnd()  &&  !itThis.isEnd() )
  {
    while ( itThis.item() < it.item() )     // walk forward until we found the
    {                                       // insertion position.
      itThis.next();
    }

    do
    {
      Sl nPos = itThis.pos();
      collection.insert( nPos, it.item() ); // insert the element
      itThis.next();                        // skip the inserted element
      it.next();
    }
    while ( !it.IsEnd()  &&  itThis.item() < it.item() );
  }

  // reached the end of one of the iterator.
  SL nPos = itThis.pos();                     // insertion position
  while ( !it.isEnd() )                       // still some items to insert ?
  {
    collection.insert( nPos, it.item() );
    it.next();
    ++nPos;
  }

  EASSERT( it.isEnd() );
}


#endif // !defined(AFX_ESORTEDCOLLECTIONT_H__32671F93_3F99_11D3_BE24_0040053AD648__INCLUDED_)
