// ESymbolIdHolderT.h: interface for the ESymbolIdHolderT class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ESYMBOLIDHOLDERT_H__992AFCF0_3D03_11D3_BE1D_0040053AD648__INCLUDED_)
#define AFX_ESYMBOLIDHOLDERT_H__992AFCF0_3D03_11D3_BE1D_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000


/* History:
 * 18/09/1999: - Creation.
 *
 * To do:
 * - When Ansi/Unicode version of EString...
 * - Add persistence.
 */


#include "EHashT.h"

/*! Id for a symbol.
 */
typedef UL ESymbolId;


/*! \class ESymbolIdHolderT<String,StringKeyProvider>
 * \include "ESymbolIdHolderT.h"
 * \brief Template to store a collection of symbol.
 *
 * @param String Character type for symbols.
 * @param StringKeyProvider Hash Key Provider for symbol string.
 *                          Should be EStringKeyProviderT<String> or
 *                                    EStringNcsKeyProviderT<String> for non case sensitive.
 */
template< class String,
          class StringKeyProvider >
class ESymbolIdHolderT  
{
public:
  typedef ::ESymbolId ESymbolId;
  typedef String ThisString;
  typedef EHashT<String,ESymbolId,const String,ESymbolId,StringKeyProvider> EHashSymbol;
  typedef EHashT<ESymbolId,const String *,ESymbolId,const String *> EHashId;

  /*! Default constructor.
   */
  ESymbolIdHolderT() : _idNext(0) 
  {
  }

  /*! Destructor.
   */
  virtual ~ESymbolIdHolderT() 
  {
  }

  /*! Build a case-sensitive symbol.
   *
   * Retreive or build a symbol. The reference that is returned is guaranted to
   * exist as long as the symbol is in the symbol holder.
   *
   * @param cszSymbol Symbol name.
   * @return Reference on the symbol.
   */
  ESymbolId symbolId( const String &strSymbol );

  /*! Get the name of a symbol of a given id.
   * @param idSymbol Id of the symbol.
   * @return Name of the symbol of id \a idSymbol.
   */
  const String &symbolName( ESymbolId idSymbol );

private:
  /*! Hash table that hold the symbols.
   */
  EHashSymbol _hashSymbol;

  /*! Associates id / symbol.
   */
  EHashId _hashId;

  /*! Last used id.
   */
  ESymbolId _idNext;
};


typedef ESymbolIdHolderT<EAString, EStringKeyProviderT<const EAString &> > EASymbolIdHolder;
typedef ESymbolIdHolderT<EAString, EStringNcsKeyProviderT<const EAString &> > EASymbolNcsIdHolder;
typedef ESymbolIdHolderT<EWString, EStringKeyProviderT<const EWString &> > EWSymbolIdHolder;
typedef ESymbolIdHolderT<EWString, EStringNcsKeyProviderT<const EWString &> > EWSymbolNcsIdHolder;
#ifdef UNICODE
  typedef EWSymbolIdHolder ESymbolIdHolder;
  typedef EWSymbolNcsIdHolder ESymbolNcsIdHolder;
#else
  typedef EASymbolIdHolder ESymbolIdHolder;
  typedef EASymbolNcsIdHolder ESymbolNcsIdHolder;
#endif


// Template implementation ESymbolIdHolderT<String,StringKeyProvider>
// -------------------------------------------------------------------

template<class String,class StringKeyProvider>
typename ESymbolIdHolderT<String,StringKeyProvider>::ESymbolId
ESymbolIdHolderT<String,StringKeyProvider>::symbolId( const String &strSymbol )
{
  const ESymbolId *pidSymbol = _hashSymbol.lookUp( strSymbol );
  if ( !pidSymbol )   // Symbol does not exist !
  {
    ESymbolId idSymbol = ++_idNext;
    _hashSymbol.add( strSymbol, 
                     idSymbol );
    _hashId.add( idSymbol, 
                 _hashSymbol.lookUpKey( strSymbol ) );
    return idSymbol;
  }
  return *pidSymbol;
}


template<class String,class StringKeyProvider>
const String &
ESymbolIdHolderT<String,StringKeyProvider>::symbolName( ESymbolId idSymbol )
{
  return *_hashId[ idSymbol ];
}

#endif // !defined(AFX_ESYMBOLIDHOLDERT_H__992AFCF0_3D03_11D3_BE1D_0040053AD648__INCLUDED_)
