// EVector3T.h: interface for the EVector3T class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EVECTOR3T_H__BE5CCCE3_6847_11D2_AEAE_C43067FFB761__INCLUDED_)
#define AFX_EVECTOR3T_H__BE5CCCE3_6847_11D2_AEAE_C43067FFB761__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

template<class EGmtVal>
class EVector3T : public EVector<3,EGmtVal>  
{
public:
  /*! Default constructor.
   */
  EVector3T() {};

  /*! Constructs a vector.
   * @param fX X Coordinate.
   * @param fY Y Coordinate.
   * @param fZ Z Coordinate.
   */
	EVector3T( EGmtVal fX,
             EGmtVal fY,
             EGmtVal fZ );

  /*! Constructs a vector from another one.
   * @param v Vector to copy.
   */
  EVector3T( const EVector3T &v );

  /*! Destructs the vector.
   */
  ~EVector3T() {};

  /*! Get vector X coordinate.
   * @return Vector X coordinate.
   */
  EGmtVal x() const;

  /*! Get vector Y coordinate.
   * @return Vector Y coordinate.
   */
  EGmtVal y() const;

  /*! Get vector Z coordinate.
   * @return Vector Z coordinate.
   */
  EGmtVal z() const;

  /*! Get vector Coordinates.
   * @param fX X Coordinate.
   * @param fY Y Coordinate.
   * @param fZ Z Coordinate.
   */
  void xyz( EGmtVal &fX,
            EGmtVal &fY,
            EGmtVal &fZ ) const;

  /*! Sets vector coordinates.
   * @param fX X Coordinate.
   * @param fY Y Coordinate.
   * @param fZ Z Coordinate.
   */
  EVector3T &set( EGmtVal fX,
                  EGmtVal fY,
                  EGmtVal fZ );

  /*! Copy operator.
   * @param v Vector to copy.
   */
  EVector3T &operator =( const EVector3T& v );


  /*! Copy the product of two vectors.
   *
   * Copy the result of the product of vector \a v3a and \a v3b to this vector.
   *
   * @param v3a First vector.
   * @param v3b Second vector.
   * @return Reference on this object.
   */
  EVector3T &product( const EVector3T &v3a,
                      const EVector3T &v3b );

private:

};


typedef EVector3T<float> EVector3;
typedef EVector3T<double> EVector3d;


template<class EGmtVal>
inline
EVector3T<EGmtVal>::EVector3T( EGmtVal fX,
                               EGmtVal fY,
                               EGmtVal fZ )
{
  set( fX, fY, fZ );
}


template<class EGmtVal>
inline
EVector3T<EGmtVal>::EVector3T( const EVector3T &v )
{
  set( v.x(), v.y(), v.z() );
}


template<class EGmtVal>
inline EGmtVal 
EVector3T<EGmtVal>::x() const
{
  return coord( 0 );
}


template<class EGmtVal>
inline EGmtVal 
EVector3T<EGmtVal>::y() const
{
  return coord( 1 );
}


template<class EGmtVal>
inline EGmtVal 
EVector3T<EGmtVal>::z() const
{
  return coord( 2 );
}


template<class EGmtVal>
inline void 
EVector3T<EGmtVal>::xyz( EGmtVal &fX,
                         EGmtVal &fY,
                         EGmtVal &fZ ) const
{
  fX = coord( 0 );
  fY = coord( 1 );
  fZ = coord( 2 );
}


template<class EGmtVal>
inline EVector3T<EGmtVal > & 
EVector3T<EGmtVal>::set( EGmtVal fX,
                         EGmtVal fY,
                         EGmtVal fZ )
{
  setCoord( 0, fX );
  setCoord( 1, fY );
  setCoord( 2, fZ );
  return *this;
}

template<class EGmtVal>
inline EVector3T<EGmtVal> &
EVector3T<EGmtVal>::operator =( const EVector3T& v )
{
  set( v.x(), v.y(), v.z() );
  return *this;
}

template<class EGmtVal>
inline EVector3T<EGmtVal> &
EVector3T<EGmtVal>::product( const EVector3T &v3A,
                             const EVector3T &v3B )
{
  set( v3A.y() * v3B.z()  -  v3B.y() * v3A.z(),
       v3A.z() * v3B.x()  -  v3B.z() * v3A.x(),
       v3A.x() * v3B.y()  -  v3B.x() * v3A.y() );
  return *this;
}

#endif // !defined(AFX_EVECTOR3T_H__BE5CCCE3_6847_11D2_AEAE_C43067FFB761__INCLUDED_)
