// EVector4.h: interface for the EVector4 class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EVECTOR4_H__5854D4D6_652F_11D2_AEA5_BCCC1933B461__INCLUDED_)
#define AFX_EVECTOR4_H__5854D4D6_652F_11D2_AEA5_BCCC1933B461__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000


template<class EGmtVal>
class EVector4T : public EVector<4,EGmtVal>  
{
public:
  /*! Default constructor.
   */
  EVector4T() {};

  /*! Constructs a vector.
   * @param fX X Coordinate.
   * @param fY Y Coordinate.
   * @param fZ Z Coordinate.
   * @param fT T Coordinate.
   */
	EVector4T( EGmtVal fX,
             EGmtVal fY,
             EGmtVal fZ,
             EGmtVal fT );

  /*! Constructs a vector from another one.
   * @param v Vector to copy.
   */
  EVector4T( const EVector4T &v );

  /*! Destructs the vector.
   */
  ~EVector4T() {};

  /*! Get vector X coordinate.
   * @return Vector X coordinate.
   */
  EGmtVal x() const;

  /*! Get vector Y coordinate.
   * @return Vector Y coordinate.
   */
  EGmtVal y() const;

  /*! Get vector Z coordinate.
   * @return Vector Z coordinate.
   */
  EGmtVal z() const;

  /*! Get vector T coordinate.
   * @return Vector T coordinate.
   */
  EGmtVal t() const;

  /*! Get vector Coordinates.
   * @param fX X Coordinate.
   * @param fY Y Coordinate.
   * @param fZ Z Coordinate.
   * @param fT T Coordinate.
   */
  void xyzt( EGmtVal &fX,
             EGmtVal &fY,
             EGmtVal &fZ,
             EGmtVal &fT ) const;

  /*! Sets vector coordinates.
   * @param fX X Coordinate.
   * @param fY Y Coordinate.
   * @param fZ Z Coordinate.
   * @param fT T Coordinate.
   */
  EVector4T &set( EGmtVal fX,
                  EGmtVal fY,
                  EGmtVal fZ,
                  EGmtVal fT );

  /*! Copy operator.
   * @param v Vector to copy.
   */
  EVector4T &operator =( const EVector4T& v );
private:

};


typedef EVector4T<float> EVector4;
typedef EVector4T<double> EVector4d;


template<class EGmtVal>
inline 
EVector4T<EGmtVal>::EVector4T( EGmtVal fX,
                               EGmtVal fY,
                               EGmtVal fZ,
                               EGmtVal fT )
{
  set( fX, fY, fZ, fT );
}


template<class EGmtVal>
inline 
EVector4T<EGmtVal>::EVector4T( const EVector4T &v )
{
  set( v.x(), v.y(), v.z(), v.t() );
}


template<class EGmtVal>
inline EGmtVal 
EVector4T<EGmtVal>::x() const
{
  return coord( 0 );
}


template<class EGmtVal>
inline EGmtVal 
EVector4T<EGmtVal>::y() const
{
  return coord( 1 );
}


template<class EGmtVal>
inline EGmtVal 
EVector4T<EGmtVal>::z() const
{
  return coord( 2 );
}


template<class EGmtVal>
inline EGmtVal 
EVector4T<EGmtVal>::t() const
{
  return coord( 3 );
}


template<class EGmtVal>
inline void 
EVector4T<EGmtVal>::xyzt( EGmtVal &fX,
                          EGmtVal &fY,
                          EGmtVal &fZ,
                          EGmtVal &fT ) const
{
  fX = coord( 0 );
  fY = coord( 1 );
  fZ = coord( 2 );
  fT = coord( 3 );
}


template<class EGmtVal>
inline EVector4T<EGmtVal > & 
EVector4T<EGmtVal>::set( EGmtVal fX,
                         EGmtVal fY,
                         EGmtVal fZ,
                         EGmtVal fT )
{
  setCoord( 0, fX );
  setCoord( 1, fY );
  setCoord( 2, fZ );
  setCoord( 3, fT );
  return *this;
}

template<class EGmtVal>
inline EVector4T<EGmtVal> &
EVector4T<EGmtVal>::operator =( const EVector4T& v )
{
  set( v.x(), v.y(), v.z(), v.t() );
  return *this;
}

#endif // !defined(AFX_EVECTOR4_H__5854D4D6_652F_11D2_AEA5_BCCC1933B461__INCLUDED_)
