// ###########################################################################
// File: ESTRING.HPP
// (c)Copyright 10/1997, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// version 0.31
// ###########################################################################


#ifndef ESTRING_HPP_INCLUDED

// Includes:
// #####################Last change: 12/10/97 13h20mn#########################
#include <string.h>
class EDataStream;

// Constantes:
// #####################Last change: 12/10/97 13h20mn#########################


// Macros:
// #####################Last change: 12/10/97 13h20mn#########################


// Types:
// #####################Last change: 12/10/97 13h20mn#########################

class EString;

// future extension: EAString -> Ansi String (8 bits).
typedef EString EAString;

// future extension: EWString -> Wide String (unicode, 16 bits).
typedef EString EWString;

class EString
{
  friend EDataStream &operator << ( EDataStream &s, const EString &str );
  friend EDataStream &operator >> ( EDataStream &s, EString &str );

  public:
    /** The null string.
     */
    static const EString nullString;

    /** Constructs a string.
     * Constructs a string with the texte \a text.
     * @param text initial string.
     */
    EString( const char *text );

    /** Copy constructor.
     * Copy string \a str.
     * @param str string to copy.
     */
    EString( const EString& str );

    /** Constructs a string.
     * Constructs a string containing character \a c.
     * @param c initial char in string.
     */
    explicit EString( char c );

    /// Default Constructor.
    EString();

    /*! Destructor.
     */
    ~EString();

    /** Copy a string.
     * Copy the string \a s of length \a nLen characters.
     * @param s string to copy
     * @param nLen length of the string (without the terminal 0).
     * @return this string.
     */
    EString& copy( const char *s,
                   SL nLen );

    /** Copy a string.
     * Copy the zero terminated string \a s.
     * @param s string to copy
     * @return this string.
     */
    EString& copy( const char *s );

    /** Copy a string.
     * Copy the string \a str.
     * @param str string to copy
     * @return this string.
     */
    EString& copy( const EString& str );

    /** Test if the string is Null.
     * Test if the string is null ( length = 0 ).
     * @return true if the string is null, false otherwise.
     */
    bool isNull() const;

    /** Set the string as a null string.
     */
    void setNull();

    /** Return the length of the string.
      * @return length of the string.
      */
    SL length() const;

    /** Return the string.
     */
    const char *datas() const;

    /** Test if the string contains a char.
     * @param c char that should be present in the string.
     * @return \c true if the string contains the char, \c false if \a c is 0 or
     *         the char is not found.
     */
    bool contains( char c ) const;

    /*! Returns the number of occurrences of the specified character in the string.
     * @param c Char which occurrences are counted.
     * @return Number of occurences of \c c in the string.
     */
    SL count( char c ) const;

    /** Get a reference on a char.
     * @param nIndex index of the char. 0 is the first.
     * @return reference on the char.
     */
    char &at( SL nIndex );

    /** Get a char.
     * @param nIndex index of the char. 0 is the first.
     * @return the char.
     */
    char at( SL nIndex ) const;

    /** Get a reference on a char.
     * @param nIndex index of the char. 0 is the first.
     * @return reference on the char.
     */
    char &operator[]( SL nIndex );

    /** Get a char.
     * @param nIndex index of the char. 0 is the first.
     * @return the char.
     */
    char operator[]( SL nIndex ) const;

    /*! Get the last char of the string.
     *
     * @return The last char of the string. \c 0 if the string is empty.
     */
    char lastChar() const;

    /** Resize the string.
     * @param nSize new size of the string.
     */
    void resize( SL nSize );

    /** Append a char to the string.
     * @param c char to append.
     * @return reference to this string.
     */
    EString& operator +=( char c );

    /** Append a string to the string.
     * @param s string to append.
     * @return reference to this string.
     */
    EString& operator +=( const char *s );

    /** Append a string to the string.
     * @param str string to append.
     * @return reference to this string.
     */
    EString& operator +=( const EString &str );

    /** Append a char to the string.
     * @param c char to append.
     * @param nCount Number of time the character is appended.
     * @return reference to this string.
     */
    EString& append( char c, SL nCount =1 );

    /** Append a string to the string.
     * @param s string to append.
     * @return reference to this string.
     */
    EString& append( const char *s );

    /** Append a string to the string.
     * @param str string to append.
     * @return reference to this string.
     */
    EString& append( const EString &str );

    /** Append a string to the string.
     * @param str string to append.
     * @param nCount Number of time the string is appended.
     * @return reference to this string.
     */
    EString& append( const EString &str,
                     SL nCount );

    /** Copy a char to the string.
     * @param c the char the string is.
     * @return reference on this string.
     */
    EString& operator =( char c );

    /** Copy a string to this string.
     * Copy the zero terminated string \a s.
     * @param s string to copy
     * @return this string.
     */
    EString& operator =( const char *s );

    /** Copy a string to this string.
     * Copy the string \a str.
     * @param str string to copy
     * @return this string.
     */
    EString& operator =( const EString& str );

    /** Concat two string.
      * @param str1 first string.
      * @param str2 second string.
      * @return a string containing the concatenation of \a str1  and \a str2.
      */
    EString operator +( const EString &str2 );

    /** Concat two string.
      * @param str1 first string.
      * @param s2 second string. If s2 is NULL then the result string is str1.
      * @return a string containing the concatenation of \a str1  and \a s2.
      */
    EString operator +( const char *s2 );

    /** Concat a string and a char.
      * @param str1 first string.
      * @param c char to concat to str1.
      * @return a string containing the concatenation of \a str1  and \a c.
      */
    EString operator +( char c );

    /** Return some of the first char of the string.
     * Copy the \a nNb first char of the string.
     * @param nNb Number of char to copy ( nNb >= 0 ).
     * @return A string with the nNb left char of the string.
     */
    EString left( SL nNb ) const;

    /** Return some of the last char of the string.
     * Copy the \a nNb last char of the string.
     * @param nNb Number of char to copy ( nNb >= 0 ).
     * @return A string with the \a nNb right char of the string.
     */
    EString right( SL nNb ) const;

    /** Return some char from the middle of the string.
     * Copy \a nNb char from index \a nIndex.
     * @param nPos Index of the first char to copy. ( nPos >= 0 ).
     * @param nNb Number of char to copy. If nNb < 0 then char are copied
     *            from nPos to length()-nNb.
     * @return A string with the some char of the string.
     */
    EString mid( SL nPos, SL nNb ) const;

    /** Insert a char in the string.
     * @param nPos position where the char is inserted. (length()>=nIndex >=0)
     * @param c char to insert.
     * @return reference on this string.
     */
    EString &insert( SL nPos, char c );

    /** Insert a string in the string.
     * @param nPos position where the string is inserted. (length()>=nIndex >=0)
     * @param s string to insert. if \a s is null then nothing is done.
     * @return reference on this string.
     */
    EString &insert( SL nPos, const char *s );

    /** Insert a string in the string.
     * @param nPos position where the string is inserted. (length()>=nIndex >=0)
     * @param str string to insert. str must be different from 'this' !
     * @return reference on this string.
     */
    EString &insert( SL nPos, const EString& str );

    /** Delete some char in the string.
     * @param nPos position of the character to delete.
     *             If nPos > length() then nothing is done.
     * @param nNb number of chars to delete. if nNb < 0 then all char are
     *            remove to position length()+nNb. A value of -1 for nNb will
     *            remove all char until the end of the string.
     * @return reference on this string.
     */
    EString &remove( SL nPos, SL nNb =1);

    /** Truncate the string.
     * @param nPos position at which the string is truncated.
     * @return reference on this string.
     */
    EString &truncate( SL nPos );

    /** Search the first occurrence of a char in the string.
     * @param c char to search for.
     * @return index of the occurrence in the string. -1 if none found.
     */
    SL pos( char c ) const;

    /** Search the first occurrence of a string in the string.
     * @param strToFind String to find.
     * @return Index of the first occurrence in the string. -1 if none found.
     */
    SL pos( const EString &strToFind ) const;

    /** Search the first occurrence of a char starting at a given position.
     * @param nPos position the search begin at. nPos >= 0.
     * @param c char to search for.
     * @return index of the occurrence in the string. -1 if none found.
     */
    SL pos( SL nPos, char c ) const;

    /** Search the first occurrence of a string in the string starting at a given position.
     * @param nPos position the search begin at. nPos >= 0.
     * @param strToFind String to find.
     * @return Index of the first occurrence in the string. -1 if none found.
     */
    SL pos( SL nPos, const EString &strToFind ) const;

    /** Search the last occurrence of a char in the string.
     * @param c char to search for.
     * @param nPosStart Position after the first char to test. If \a nPosStart =
     *                  length() then the search begin from the end of the string.
     *                  If negative then length() is added. Therefore, -1 will begin
     *                  the search from the end of the string...
     * @return index of the occurrence in the string. -1 if none found.
     */
    SL posLast( char c, 
                SL nPosStart =-1 ) const;

    /** Test if the string is a given char.
     * @param c char we compare the string to.
     * @return true if the string is the given char, false otherwise.
     */
    bool operator ==( char c ) const;

    /** Test if this string is different to another one.
     * @param str string this string is compared to.
     * @return true if the strings are different, false if they are equal.
     */
    bool operator !=( const EString &str ) const;

    /** Test if this string is different to another one.
     * @param s string this string is compared to.
     * @return true if the strings are different, false if they are equal.
     */
    bool operator !=( const char *s ) const;

    /** Test if the string is not a given char.
     * @param c char we compare the string to.
     * @return true if the string is not the given char, false otherwise.
     */
    bool operator !=( char c ) const;

    /** Test if this string is less than another one.
     * @param str string this string is compared to.
     * @return true if this string is less than the string \a str.
     */
    bool operator <( const EString &str ) const;

    /** Test if this string is less than another one.
     * @param s string this string is compared to.
     * @return true if this string is less than the string \a s.
     */
    bool operator <( const char *s ) const;

    /** Test if this string is less than a char.
     * @param c char the string is compared to.
     * @return true if this string is less than the char \a c.
     */
    bool operator <( char c ) const;

    /** Test if this string is less or equal to another one.
     * @param str string this string is compared to.
     * @return true if this string is less or equal to the string \a str.
     */
    bool operator <=( const EString &str ) const;

    /** Test if this string is less or equal to another one.
     * @param s string this string is compared to.
     * @return true if this string is less or equal to the string \a str.
     */
    bool operator <=( const char *s ) const;

    /** Test if this string is less or equal to a char.
     * @param c char the string is compared to.
     * @return true if this string is less or equal to the char \a c.
     */
    bool operator <=( char c ) const;

    /** Test if this string is greater than another one.
     * @param str string this string is compared to.
     * @return true if this string is greater than the string \a str.
     */
    bool operator >( const EString &str ) const;

    /** Test if this string is greater than another one.
     * @param s string this string is compared to.
     * @return true if this string is greater than the string \a s.
     */
    bool operator >( const char *s ) const;

    /** Test if this string is greater than a char.
     * @param c char the string is compared to.
     * @return true if this string is greater than the char \a c.
     */
    bool operator >( char c ) const;

    /** Test if this string is greater or equal to another one.
     * @param str string this string is compared to.
     * @return true if this string is greater or equal to the string \a str.
     */
    bool operator >=( const EString &str ) const;

    /** Test if this string is greater or equal to another one.
     * @param s string this string is compared to.
     * @return true if this string is greater or equal to the string \a s.
     */
    bool operator >=( const char *s ) const;

    /** Test if this string is greater or equal to a char.
     * @param c char the string is compared to.
     * @return true if this string is greater or equal to the char \a c.
     */
    bool operator >=( char c ) const;

    /** Compare this string to another string.
     * This is a case sensitive compare.
     * @param str string this string is compared to.
     * @return 0 is the 2 string are equal, or both null.
     *         >0 if this string is greater than \a str.
     *         <0 if this string if less than \a str.
     */
    SL cmp( const EString& str ) const;

    /** Compare this string to another string.
     * This is a case sensitive compare.
     * @param s string this string is compared to.
     * @return 0 is the 2 string are equal, or both null.
     *         >0 if this string is greater than \a s.
     *         <0 if this string if less than \a s.
     */
    SL cmp( const char *s ) const;

    /** Compare this string to a char.
     * This is a case sensitive compare.
     * @param c char the string is compared to.
     * @return 0 is the string as a length of 1 and the first char is equal to
     *           \a c.
     *         >0 if this string is greater than c.
     *         <0 if this string if less than c.
     */
    SL cmp( char c ) const;

    /** Compare this string to another string.
     * This is a case insensitive compare.
     * @param str string this string is compared to.
     * @return 0 is the 2 string are equal, or both null.
     *         >0 if this string is greater than \a str.
     *         <0 if this string if less than \a str.
     */
    SL icmp( const EString& str ) const;

    /** Compare this string to another string.
     * This is a case insensitive compare.
     * @param s string this string is compared to.
     * @return 0 is the 2 string are equal, or both null.
     *         >0 if this string is greater than \a s.
     *         <0 if this string if less than \a s.
     */
    SL icmp( const char *s ) const;

    /** Compare this string to a char.
     * This is a case insensitive compare.
     * @param c char the string is compared to.
     * @return 0 is the string as a length of 1 and the first char is equal to
     *           \a c.
     *         >0 if this string is greater than c.
     *         <0 if this string if less than c.
     */
    SL icmp( char c ) const;

    /** Compare a given number of chars in the two string.
     * @param str string this string is compared to.
     * @param nNb number of chars to compare (nNb >= 0).
     * @return 0 is the \a nNb first chars of both string are equal.
     *         >0 if this string is greater than \a str.
     *         <0 if this string if less than \a str.
     */
    SL cmpn( const EString &str, SL nNb ) const;

    /** Compare a given number of chars in the two string.
     * @param s string this string is compared to.
     * @param nNb number of chars to compare (nNb >= 0).
     * @return 0 is the \a nNb first chars of both string are equal.
     *         >0 if this string is greater than \a s.
     *         <0 if this string if less than \a s.
     */
    SL cmpn( const char *s, SL nNb ) const;

    /** Get the string.
     * @param the string, in a C string format.
     */
    const char *cstr() const;

    /** Get a copy of the string where all char are in lower case.
     * @return a copy of the string in lower case.
     */
    EString lower() const;

    /** Get a copy of the string where all char are in upper case.
     * @return a copy of the string in upper case.
     */
    EString upper() const;

    /** Convert all char of the string to lower case.
     * @return reference on this string.
     */
    EString &toLower();

    /** Convert all char of the string to upper case.
     * @return reference on this string.
     */
    EString &toUpper();

    /** Get the string buffer.
     * Get the string buffer (for easier change).
     * @return the string buffer.
     */
    const char *string() const;

    /** Make a case sensitive key for hashtable.
     * The key is a numeric value build from the string content.
     * @return key value.
     */
    UL makeHashKey() const;

    /** Make a case insensitive key for hashtable.
     * The key is a numeric value build from the string content. The key
     * returned is computed by applying tolower on each char. Therefore
     * the key value is the same for a upper case and a lower case string.
     * @return key value.
     */
    UL imakeHashKey() const;

	  /** Put the string inside double quotes.
	   * If they are some quote in the string then they are quoted.
	   * For example the text Toto"Baba would be returned as "Toto"""Baba".
	   * @param strText text to quote.
	   * @return quoted text.
	   */
    EString dquoted() const;

	  /** Put the string inside quotes.
	   * If they are some quote in the string then they are quoted.
	   * For example the text Toto"Baba would be returned as 'Toto'''Baba'.
	   * @param strText text to quote.
	   * @return quoted text.
	   */
    EString quoted() const;

    /*! Tests if the string is a positive integer.
     * @return \c true if the string is a positive interger. \c false if it is not.
     */
    bool isInt() const;

    /*! Tests if the string is a signed integer.
     * 
     * The integer may begin with '+'or '-'.
     *
     * @return \c true if the string is a signed interger. \c false if it is not.
     */
    bool isSignedInt() const;

    /*! Get the signed integer value of the string.
     * @return Signed integer value of the string. If the string is not an integer
     *         0 is returned.
     */
    SL valSignedInt() const;

    /*! Get the unsigned integer value of the string.
     * @return Unsigned integer value of the string. If the string is not an integer
     *         0 is returned.
     */
    UL valInt() const;

    /*! Tests if the string is a hexadecimal integer.
     * 
     * Tests if the string is only made of char 0..9 and a..f.
     *
     * @return \c true if the string is a hexadecimal integer, \c false if it is not.
     */
    bool isHexaInt() const;

    /*! Get the hexadecimal integer value of the string.
     * @return Unsigned integer value of the string. If the string is not an integer
     *         0 is returned.
     */
    UL valHexaInt() const;

    /*! Trim space char at the beginning of the string.
     * @return Reference on this object.
     */
    EString &trimLeft();

    /*! Trim space char at the end of the string.
     * @return Reference on this object.
     */
    EString &trimRight();

    /*! Trim space char at the beginning and at the end of the string.
     * @return Reference on this object.
     */
    EString &trimSpaces();

    /*! Set the string to a hexadecimal representation of a number.
     *
     * Example of use:
     *
     * setHexa( 0x12345678, NULL, 4, 0 ) will output "5678".
     * setHexa( 0x12345678, NULL, 2, 4 ) will output "34".
     *
     * @param nNum Number to write in hexa.
     * @param sPrefix Prefix to the hexadecimal code. If \c NULL then no prefix
     *                are use.
     * @param nNumDigit Number of digit to write.
     * @param nLowerDigit Zero based of the lower digit to write. 0 is the rightest 
     *                    digit.
     */
    void setHexa( UL nNum,
                  const char *sPrefix =NULL,
                  SL nNumDigit = (sizeof(UL)/8),
                  SL nLowerDigitNo = 0 );

    /*! Tests if the string is a Unicode string.
     * @return \c true if the string is Unicode, \c false if the string is Unicode.
     */
    bool isUnicode() const;

    /*! Tests if the string is a Ansi string.
     * @return \c true if the string is Ansi, \c false if the string is Unicode.
     */
    bool isAnsi() const;

    /*! Get the length of the string in byte including the terminal 0.
     * @return Length of the string in byte including the terminal 0.
     */
    SL byteLength() const;

    /*! Ensures that the string is not NULL!.
     * If the string is NULL (length of 0), then the buffer is reallocated
     * so that the string is an empty string (terminal 0).
     *
     * This method is to be used when the terminal 0 is needed.
     */
    void ensureNotNull();

    /*! Format the string like an sprintf;
     * @param szFormat Format control string.
     */
    void format( const char *szFormat,
                 ... );

protected:
    /// initialize members.
    void init();

    /** Get the string buffer.
     * Get the string buffer (for easier change).
     * @return the string buffer.
     */
    char *string();

    /// Buffer.
    char *_sString;

    /// String length.
    SL _nLength;

private:
    /// By how many chars the buffer grow each time it needs to.
    static const SL _nGranularity;

};

// Prototypes:
// ###########################################################################

EDataStream &operator << ( EDataStream &s, const EString &str );

EDataStream &operator >> ( EDataStream &s, EString &str );

// inlines:
// #####################Last change: 12/10/97 13h20mn#########################


/** Get the string buffer.
  * Get the string buffer (for easier change).
  * @return the string buffer.
  */
inline char *EString::string()
{
  return _sString;
}

/** Get the string buffer.
  * Get the string buffer (for easier change).
  * @return the string buffer.
  */
inline const char *EString::string() const
{
  return _sString;
}

/// Default Constructor.
inline EString::EString()
{
  init();
}

/** Constructs a string.
  * Constructs a string with the texte \a text.
  * @param text initial string.
  */
inline EString::EString( const char *text )
{
  init();
  copy( text );
}

/** Copy constructor.
  * Copy string \a s. It is equivalent to duplicate( s );
  * @param s string to copy.
  */
inline EString::EString( const EString& str )
{
  init();
  copy( str );
}

inline
EString::~EString()
{
  efree( _sString );
}


/** Constructs a string.
  * Constructs a string containing character \a c.
  * @param c initial char in string.
  */
inline EString::EString( char c )
{
  init();
  resize(1);
  at(0) = c;
}

/** Test if the string is Null.
  * Test if the string is null ( length = 0 ).
  * @return true if the string is null, false otherwise.
  */
inline bool EString::isNull() const
{
  return length() == 0;
}

/** Set the string as a null string.
  */
inline void EString::setNull()
{
  resize( 0 );
}

/** Return the length of the string.
  * @return length of the string.
  */
inline SL EString::length() const
{
  return _nLength;
}

/** Return the string.
  */
inline const char *EString::datas() const
{
  return string();
}

/** Test if the string contains a char.
  * @param c char that should be present in the string.
  * @return true if the string contains the char.
  */
inline bool EString::contains( char c ) const
{
  return (string()  &&  c != 0) ? (strchr( string(), c ) != ENIL) : false;
}


/** Append a char to the string.
  * @param c char to append.
  * @return reference to this string.
  */
inline EString& EString::operator +=( char c )
{
  SL nLen = length();
  resize( nLen +1 );
  at( nLen ) = c;
  return *this;
}


/** Get a reference on a char.
  * @param nIndex index of the char. 0 is the first.
  * @return reference on the char.
  */
inline char &EString::at( SL nIndex )
{
  EASSERT( nIndex >= 0 );
  EASSERT( nIndex < _nLength );
  EASSERT( string() );
  return string()[ nIndex ];
}

/** Get a char.
  * @param nIndex index of the char. 0 is the first.
  * @return the char.
  */
inline char EString::at( SL nIndex ) const
{
  EASSERT( nIndex >= 0 );
  EASSERT( nIndex < _nLength );
  EASSERT( string() );
  return string()[ nIndex ];
}

/** Get a reference on a char.
  * @param nIndex index of the char. 0 is the first.
  * @return reference on the char.
  */
inline char &EString::operator[]( SL nIndex )
{
  EASSERT( nIndex >= 0 );
  EASSERT( nIndex < _nLength );
  EASSERT( string() );
  return string()[ nIndex ];
}

/** Get a char.
  * @param nIndex index of the char. 0 is the first.
  * @return the char.
  */
inline char EString::operator[]( SL nIndex ) const
{
  EASSERT( nIndex >= 0 );
  EASSERT( nIndex < _nLength );
  EASSERT( string() );
  return string()[ nIndex ];
}

/** Copy a string to this string.
  * Copy the zero terminated string \a s.
  * @param s string to copy
  * @return this string.
  */
inline EString& EString::operator =( const char *s )
{
  return copy( s );
}

/** Copy a string to this string.
  * Copy the string \a str.
  * @param str string to copy
  * @return this string.
  */
inline EString& EString::operator =( const EString& str )
{
  return copy( str );
}

/** Copy a char to the string.
  * @param c the char the string is.
  * @return reference on this string.
  */
inline EString& EString::operator =( char c )
{
  resize( 1 );
  at( 0 ) = c;
  return *this;
}


inline EString& 
EString::append( char c, SL nCount )
{
  while ( nCount-- > 0 )
    *this += c;
  return *this;
}

/** Append a string to the string.
  * @param s string to append.
  * @return reference to this string.
  */
inline EString& EString::append( const char *s )
{
  return *this += s;
}

/** Append a string to the string.
  * @param str string to append.
  * @return reference to this string.
  */
inline EString& EString::append( const EString &str )
{
  return *this += str;
}

/** Truncate the string.
  * @param nPos position at which the string is truncated.
  * @return reference on this string.
  */
inline EString &EString::truncate( SL nPos )
{
  return remove( nPos, -1 );
}

inline bool operator ==( const EString &str1,
                         const EString &str2 )
{
  return str1.cmp( str2 ) == 0;
}

inline bool operator ==( const EString &str,
                         const char *s )
{
  return str.cmp( s ) == 0;
}

inline bool EString::operator ==( char c ) const
{
  return cmp( c ) == 0;
}

inline bool EString::operator !=( const EString &str ) const
{
  return cmp( str ) != 0;
}

inline bool EString::operator !=( const char *s ) const
{
  return cmp( s ) != 0;
}

inline bool EString::operator !=( char c ) const
{
  return cmp( c ) != 0;
}

inline bool EString::operator <( const EString &str ) const
{
  return cmp( str ) < 0;
}

inline bool EString::operator <( const char *s ) const
{
  return cmp( s ) < 0;
}

inline bool EString::operator <( char c ) const
{
  return cmp( c ) < 0;
}

inline bool EString::operator <=( const EString &str ) const
{
  return cmp( str ) <= 0;
}

inline bool EString::operator <=( const char *s ) const
{
  return cmp( s ) <= 0;
}

inline bool EString::operator <=( char c ) const
{
  return cmp( c ) <= 0;
}

inline bool EString::operator >( const EString &str ) const
{
  return cmp( str ) > 0;
}

inline bool EString::operator >( const char *s ) const
{
  return cmp( s ) > 0;
}

inline bool EString::operator >( char c ) const
{
  return cmp( c ) > 0;
}


inline bool EString::operator >=( const EString &str ) const
{
  return cmp( str ) >= 0;
}

inline bool EString::operator >=( const char *s ) const
{
  return cmp( s ) >= 0;
}

inline bool EString::operator >=( char c ) const
{
  return cmp( c ) >= 0;
}

/** Get the string.
  * @param the string, in a C string format.
  */
inline const char *
EString::cstr() const
{
  return string();
}

#ifdef __WATCOMC__
/** Convert all char of the string to lower case.
  * @return reference on this string.
  */
inline EString &EString::toLower()
{
  strlwr( string() );
  return *this;
}

/** Convert all char of the string to upper case.
  * @return reference on this string.
  */
inline EString &EString::toUpper()
{
  strupr( string() );
  return *this;
}
#endif


inline char 
EString::lastChar() const
{
  return length() ? at( length() -1 ) : 0;
}


inline EString 
operator +( const EString &str1, 
            const EString &str2 )
{
  EString str( str1 );
  str += str2;
  return str;
}


inline EString 
operator +( const EString &str1, 
            const TCHAR *s2 )
{
  EString str( str1 );
  str += s2;
  return str;
}


inline EString 
operator +( const EString &str1, 
            TCHAR c )
{
  EString str( str1 );
  str += c;
  return str;
}


inline bool 
EString::isUnicode() const
{
  return false;
}


inline bool 
EString::isAnsi() const
{
  return true;
}


inline SL 
EString::byteLength() const
{
  return (length() +1) * sizeof( TCHAR );
}


inline SL 
EString::pos( const EString &strToFind ) const
{
  return pos( 0, strToFind );
}


// Prototypes:
// #####################Last change: 12/10/97 13h20mn#########################



#define ESTRING_HPP_INCLUDED
#endif


// ########################### EOF ESTRING.HPP ###############################
// ########################### EOF ESTRING.HPP ###############################
// ########################### EOF ESTRING.HPP ###############################

