// ###########################################################################
// File: ETINYSTR.HPP
// (c)Copyright 03/1998, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// ###########################################################################


#ifndef ETINYSTR_HPP_INCLUDED


// Includes:
// #####################Last change: 29/03/98 10h58mn#########################
#include "estring.hpp"

// Constantes:
// #####################Last change: 29/03/98 10h58mn#########################


// Macros:
// #####################Last change: 29/03/98 10h58mn#########################


// Types:
// #####################Last change: 29/03/98 10h58mn#########################

class ETinyString
{
public:
  /** Constructs a tiny string from a string.
   * The string is not duplicated and should not be changed.
   */
  ETinyString( const EString& strString );

  /** Constructs a tiny string from a string.
   * The string is not duplicated and should not be changed.
   */
  ETinyString( char *sString );

  /// Default constructor.
  ETinyString() {};

  /** Detach the string.
   * If the string was initialized from another string, the data are duplicated.
   */
  void detach();

  /** Get a reference on a char.
   */
  char& operator[]( SL nPos );

  /** Get a reference on a char.
   */
  char operator[]( SL nPos ) const;

  /** Compare to another string.
   * @param sString string this string is compared to.
   * @return true if the string are equal.
   */
  bool operator ==( const ETinyString& sString ) const;

  /** Case insensitive compare to another string.
   * @param sString string this string is compared to.
   * @return true if the string are equal.
   */
  bool icmp( const ETinyString& sString ) const;

  /** Copy operator.
   * The string is not detached for copy!
   * @param sString string to copy.
   * @return return on this string.
   */
  ETinyString& operator =(const ETinyString& sString );


  operator const char * () const;

  /** Convert string to lower case.
   * @return reference on this string.
   */
  ETinyString& toLower();

  /** Convert string to upper case.
   * @return reference on this string.
   */
  ETinyString& toUpper();


private:
  /// String.
  char *_sString;
};

// inline:
// #####################Last change: 29/03/98 10h58mn#########################

inline ETinyString::ETinyString( const EString& strString )
{
  _sString = (char *)strString.datas();
}

inline ETinyString::ETinyString( char *sString )
{
  _sString = sString;
}

inline void ETinyString::detach()
{
  _sString = _strdup( _sString );
}

inline char& ETinyString::operator[]( SL nPos )
{
  return _sString[ nPos ];
}

inline char ETinyString::operator[]( SL nPos ) const
{
  return _sString[ nPos ];
}

inline bool ETinyString::operator ==( const ETinyString& sString ) const
{
  return !strcmp( _sString, sString._sString );
}

inline bool ETinyString::icmp( const ETinyString& sString ) const
{
  return !_stricmp( _sString, sString._sString );
}

inline ETinyString& ETinyString::operator =(const ETinyString& sString )
{
  _sString = sString._sString;
  return *this;
}

inline ETinyString::operator const char * () const
{
  return _sString;
}

inline ETinyString& ETinyString::toLower()
{
  _strlwr( _sString );
  return *this;
}

inline ETinyString& ETinyString::toUpper()
{
  _strupr( _sString );
  return *this;
}


// Prototypes:
// #####################Last change: 29/03/98 10h58mn#########################



#define ETINYSTR_HPP_INCLUDED
#endif


// ########################### EOF ETINYSTR.HPP ##############################
// ########################### EOF ETINYSTR.HPP ##############################
// ########################### EOF ETINYSTR.HPP ##############################

