// EwgProgressInfo.h: SLerface for the EwgProgressInfo class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EWGPROGRESSINFO_H__2E202D03_831F_11D2_AEE8_840A6D805765__INCLUDED_)
#define AFX_EWGPROGRESSINFO_H__2E202D03_831F_11D2_AEE8_840A6D805765__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

class EwgProgressInfo  
{
public:
	EwgProgressInfo();
  virtual ~EwgProgressInfo() {}

  EwgProgressInfo &setProgressRange( SL nMin, 
                                     SL nMax );

  EwgProgressInfo &setProgressValue( SL nValue );

  EwgProgressInfo &startProgress();

  EwgProgressInfo &stopProgress();

  SL minProgressValue() const;
  SL maxProgressValue() const;
  SL progressValue() const;

  /** Tests if the progress has been started.
   * 
   * The progress is started when _nTickCountToStart milliseconds has ellapsed.
   */
  bool isProgressStarted() const;

  /** Set the delay between each progress update.
   */
  void setProgressUpdateDelay( UL ulUpdateDelay );

  /** Set the time that must ellapse before the progress is started.
    */
  void setProgressStartDelay( UL ulStartDelay );

  virtual void onProgressStart() {};
  virtual void onProgress() {};
  virtual void onProgressStop() {};


private:
  /** Number of milliseconds to ellapse before the progress is started.
   */
  UL _ulTickCountToStart;

  /** Tick count value when the progress was last updated.
   */
  UL _ulTickCountLast;

  /** Number of millisecond between each update.
   */
  UL _ulTickCountToUpdate;

  SL _nMinValue;
  SL _nMaxValue;
  SL _nValue;
  SL _nValueShift;      // right shift to keep value 16 bits (WIN32 is not 32 bits!!!)

  /** Is the progress started ?
   * @see isProgressStarted().
   */
  bool _bIsStarted;
};





inline EwgProgressInfo &
EwgProgressInfo::startProgress()
{
  _nValue = _nMinValue;
  _ulTickCountLast = GetTickCount();
  return *this;
}


inline EwgProgressInfo &
EwgProgressInfo::stopProgress()
{
  if ( isProgressStarted() )
  {
    _bIsStarted =false;
    onProgressStop();
  }
  return *this;
}


inline SL 
EwgProgressInfo::minProgressValue() const
{
  return 0;
}


inline SL 
EwgProgressInfo::maxProgressValue() const
{
  return (_nMaxValue - _nMinValue) >> _nValueShift;
}


inline SL 
EwgProgressInfo::progressValue() const
{
  return _nValue;
}


inline bool 
EwgProgressInfo::isProgressStarted() const
{
  return _bIsStarted;
}


inline void 
EwgProgressInfo::setProgressUpdateDelay( UL ulUpdateDelay )
{
  _ulTickCountToUpdate = ulUpdateDelay;
}


inline void 
EwgProgressInfo::setProgressStartDelay( UL ulStartDelay )
{
  _ulTickCountToStart = ulStartDelay;
}

#endif // !defined(AFX_EWGPROGRESSINFO_H__2E202D03_831F_11D2_AEE8_840A6D805765__INCLUDED_)
