// //////////////////////////////////////////////////////////////////////////
// Header file Assert.h for class Assert
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2001/10/10
// //////////////////////////////////////////////////////////////////////////
#ifndef GAIACRNT_ASSERT_H
#define GAIACRNT_ASSERT_H

#include <string>
#include <strstream>
#include <stdexcept>
#include <cppunit/Asserter.h>

class Stringable;

namespace Assert
{

template<typename TheType>
struct stringizer
{

  struct Holder
  {
    Holder( const TheType &value ) :
      _value( value )
    {
    }

    std::string stringable( const Stringable &)
    {
      return const_cast<TheType &>(_value).toString();
    }

    std::string stringable( ... )
    {
      std::ostringstream stream;
      stream << _value;
      return stream.str();
    }

    const TheType &_value;
  };

  static std::string toString( const TheType &value )
  {
    return Holder(value).stringable( value );
  }
};


template<typename TheType1, typename TheType2>
struct equalizer
{
  static bool equals( const TheType1 &a, const TheType2 &b )
  {
    return a == b;
  }
};


template<typename TheType1, typename TheType2>
void 
checkEquals( const TheType1 &a,
             const TheType2 &b,
             CppUnit::SourceLine sourceLine )
{
  if ( equalizer<TheType1,TheType2>::equals( a, b ) )
    return;
  CppUnit::Asserter::failNotEqual( toString( a ),
                                   toString( b ),
                                   sourceLine );
}


}  // namespace Assert


template<typename TheType>
std::string
toString( const TheType &v )
{
  return Assert::stringizer<TheType>::toString( v );
}


#define GAIACRTN_ASSERT_EQUAL( expected, actual ) \
    Assert::checkEquals( expected, actual, CPPUNIT_SOURCELINE() )


#endif  // GAIACRNT_ASSERT_H
