// TtrEnclosure.h: interface for the TtrEnclosure class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TTRENCLOSURE_H__4129354D_36AF_473D_9327_546078966C20__INCLUDED_)
#define AFX_TTRENCLOSURE_H__4129354D_36AF_473D_9327_546078966C20__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

class TtrEnclosure;

/*!
 * Iterator on a list of pointer on TtrEnclosure.
 */
typedef ERandomItT<TtrEnclosure *> ItTtrEnclosure;

/*!
 * \brief This class represents an enclosure.
 * An enclosure is a set of two string that enclose a text, such as  ( ) or [ ]. 
 * Usually, such enclosure prevents that paragraph may occur in the text within.
 */
class TtrEnclosure  
{
public:
  /*!
   * Constructs an enclosure.
   * \a strOpen and \a strClose can not be empty string.
   * @param strOpen String that "opens" the enclosure (such as "(" for example).
   * @param strClose String that "closes" the enclosure (such as ")" for example).
   */
	TtrEnclosure( const EString &strOpen,
                const EString &strClose );

  /*!
   * Destructor.
   */
	virtual ~TtrEnclosure();

  const EString &getOpening() const;
  const EString &getClosure() const;

  bool operator ==( const TtrEnclosure &enclosure ) const;
  bool operator !=( const TtrEnclosure &enclosure ) const;

private:
  EString _strOpen;
  EString _strClose;
};


#endif // !defined(AFX_TTRENCLOSURE_H__4129354D_36AF_473D_9327_546078966C20__INCLUDED_)
