// //////////////////////////////////////////////////////////////////////////
// Header file TtrFmtBlank.h for class TtrFmtBlank
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/15
// //////////////////////////////////////////////////////////////////////////
#ifndef TTRFMTBLANK_H
#define TTRFMTBLANK_H

#include <deque>
#include "TtrFmtElt.h"
#include "TtrFmtIndentInfo.h"
#include "TtrTextLineFormat.h"


/*! \class TtrFmtBlank
 * \brief This class represents a blank block of a formatted text.
 */
class TtrFmtBlank : public TtrFmtElt
{
public:
  /*! Constructs a TtrFmtBlank object.
   */
  TtrFmtBlank();

  /*! Constructs a TtrFmtBlank object.
   */
  TtrFmtBlank( const TtrFmtIndentInfo &infoIndent,
               const TtrTextLineFormat &format );

  /*! Destructor.
   */
  virtual ~TtrFmtBlank();

  /*! Gets the type of the element.
   * @return Type of the element.
   */
  virtual EltType type() const;

  /*! Visits the element.
   * @param visitor Visitor that is visited by the element.
   */
  virtual void visit( TtrFmtEltVisitor &visitor );

  /*! Sets the indent information.
   * @param infoIndent Indent information.
   */
  void setIndent( const TtrFmtIndentInfo &infoIndent );

  /*! Adds a line of blank.
   * @param format Format of the new block.
   */
  void addLine( const TtrTextLineFormat &format );

  /*! Returns the number of blank lines in the block.
   * @return Number of blank line in the block.
   */
  SL lineCount() const;

  bool operator ==( const TtrFmtBlank &cmpTo ) const;
  bool operator !=( const TtrFmtBlank &cmpTo ) const;

private:
  /*! Prevents the use of the copy constructor.
   */
  TtrFmtBlank( const TtrFmtBlank &copy );

  /*! Prevents the use of the copy operator.
   */
  TtrFmtBlank &operator =( const TtrFmtBlank &copy );

private:
  TtrFmtIndentInfo _infoIndent;
  std::deque<TtrTextLineFormat> _formats;
};



// Inlines methods for TtrFmtBlank:
// --------------------------------


inline void 
TtrFmtBlank::setIndent( const TtrFmtIndentInfo &infoIndent )
{
  _infoIndent = infoIndent;
}


#endif  // TTRFMTBLANK_H
