// //////////////////////////////////////////////////////////////////////////
// Header file TtrUtReformatter.h for class TtrUtReformatter
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2001/01/17
// //////////////////////////////////////////////////////////////////////////
#ifndef TTRUTREFORMATTER_H
#define TTRUTREFORMATTER_H

#include "CppUnit.h"
#include "TtrTextRaw.h"
#include "TtrTextReformatterBuilder.h"

class TtrFmtText;
class TtrTextReformatter;

/*! \class TtrUtReformatter
 * \brief This class unit tests the Reformatter process.
 */
class TtrUtReformatter : public CppUnit::TestCase,
                         public TtrTextReformatterBuilder
{
  CPPUNIT_TEST_SUITE( TtrUtReformatter );
  CPPUNIT_TEST( testBuildTextBlocks );
  CPPUNIT_TEST( testChangeLevel );
  CPPUNIT_TEST( testIsGluedAndPrefixedLineChangingLevel );
  CPPUNIT_TEST( testIsIndentedLineChangingLevel );
  CPPUNIT_TEST( testIsIndentedLineChangingLevelTo0 );
  CPPUNIT_TEST( testStartNewBlock );
  CPPUNIT_TEST( testContinueBlock );
  CPPUNIT_TEST( testStartNewBlockPreservingIndentation );
  CPPUNIT_TEST( testStartNewIndentedBlock );
  CPPUNIT_TEST( testEndBlock );
  CPPUNIT_TEST( testIsIndentedLineStartingNewBlock );
  CPPUNIT_TEST( testIsIndentedLineWithPrefixStartingNewBlock );
  CPPUNIT_TEST( testIsPrefixedLineStartingNewBlock );
  CPPUNIT_TEST( testIsGluedLineStartingNewBlock );
  CPPUNIT_TEST( testIsGluedLineWithPrefixStartingNewBlock );
  CPPUNIT_TEST_SUITE_END();

public:
  /*! Constructs a TtrUtReformatter object.
   */
  TtrUtReformatter();

  /*! Destructor.
   */
  virtual ~TtrUtReformatter();

  virtual void setUp();
  virtual void tearDown();

  void testBuildTextBlocks();
  void testChangeLevel();
  void testIsGluedAndPrefixedLineChangingLevel();
  void testIsIndentedLineChangingLevel();
  void testIsIndentedLineChangingLevelTo0();
  void testStartNewBlock();
  void testContinueBlock();
  void testStartNewBlockPreservingIndentation();
  void testStartNewIndentedBlock();
  void testEndBlock();
  void testIsIndentedLineStartingNewBlock();
  void testIsIndentedLineWithPrefixStartingNewBlock();
  void testIsPrefixedLineStartingNewBlock();
  void testIsGluedLineStartingNewBlock();
  void testIsGluedLineWithPrefixStartingNewBlock();

private:

  virtual void changeLevel( SL nBlockLevel,
                            const EString &strLevelPrefix );

  virtual void newBlock( const EString &strContent,
                         SL nIndentCount,
                         const EString &strIndent,
                         const TtrTextLineFormat &format );

  virtual void appendToBlock( const EString &strTextToAppend );

  virtual void endBlock();

private:
  class TtrTextBuilder;

  class Root {
  public:
    virtual ~Root() {}
  };
  
  class ChangeLevelParams : public Root {
  public:
    ChangeLevelParams( SL nBlockLevel, 
                       EString strLevelPrefix ) :
        _nBlockLevel( nBlockLevel ),
        _strLevelPrefix( strLevelPrefix ) {}
    SL _nBlockLevel;
    EString _strLevelPrefix;
  };
  
  class NewBlockParams : public Root {
  public:
    NewBlockParams( EString strContent, 
                    SL nIndentCount,
                    EString strIndent,
                    const TtrTextLineFormat &format ) :
        _strContent( strContent ),
        _nIndentCount( nIndentCount ),
        _strIndent( strIndent ),
        _format( format ) {}
    EString _strContent;
    SL _nIndentCount;
    EString _strIndent;
    TtrTextLineFormat _format;
  };

  class AppendToBlockParams : public Root {
  public:
    AppendToBlockParams( EString strTextToAppend ) :
        m_strTextToAppend( strTextToAppend ) {}
    EString m_strTextToAppend;
  };

  class EndBlockParams : public Root {
  };

private:
  template<typename ExpectedParam>
  void getNextCall( ExpectedParam **pCall )
  {
    GAIACRTN_ASSERT( _nExpectedCallIndex < _vCalls.count() );
    Root *pRootCall = _vCalls[ _nExpectedCallIndex++ ];
    *pCall = dynamic_cast<ExpectedParam *>( pRootCall );
    GAIACRTN_ASSERT( *pCall != NULL );
  }

  void expectCall( Root *pCall );
  void checkNoRemainingCall();

  TtrUtReformatter( const TtrUtReformatter &copy );

  TtrUtReformatter &operator =( const TtrUtReformatter &copy );

private:
  TtrTextRaw _text;
  
  typedef EPtrDynArrayT<Root *> ExpectedCalls;
  ExpectedCalls _vCalls;
  SL _nExpectedCallIndex;

  TtrTextLineFormat _formatNone;
  TtrTextLineFormat _formatCenter;
  
  TtrTextReformatter *_pReformatter;
};



// Inlines methods for TtrUtReformatter:
// -------------------------------------



#endif  // TTRUTREFORMATTER_H
