// //////////////////////////////////////////////////////////////////////////
// Implementation file TtrUtWriterWrappedTxt.cpp for class TtrUtWriterWrappedTxt
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2001/05/06
// //////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "TtrUtWriterWrappedTxt.h"
#include "TtrFmtPara.h"
#include "TtrGlobalSetting.h"
#include "TtrTagDef.h"
#include "TtrTagManager.h"
#include "TtrWriterSetting.h"
#include "TtrWriterWrappedTxt.h"

CPPUNIT_TEST_SUITE_REGISTRATION( TtrUtWriterWrappedTxt );

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// Class TtrUtWriterWrappedTxt::WrapChecker
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------


class TtrUtWriterWrappedTxt::WrapChecker
{
public:
  WrapChecker( TtrUtWriterWrappedTxt &test ) :
      _test( test )
  {
  }

  void addLine( const EString &strLine )
  {
    if ( !_strParagraph.isNull() )
      _strParagraph += " ";
    _strParagraph += strLine;
  }

  void addExpected( const EString &strLine )
  {
    _strExpected += strLine;
    _strExpected += _test._strEol;
  }

  void check( int nIndentLevel,
              const EString &strPrefix ="",
              bool bCenter =false )
  {
    TtrFmtPara para( _strParagraph, 
                     nIndentLevel );
    TtrFmtIndentInfo indent( para.indent() );
    indent.setIndentString( strPrefix );
    para.setIndent( indent );
    TtrTextLineFormat format;
    if ( bCenter )
      format.centerOn();
    para.setFormat( format );

    _test._pWriter->visitPara( para );

//    TRACE( "Expected:\n%s\n", (LPCTSTR)_strExpected );
//    TRACE( "Was:\n%s\n", (LPCTSTR)_test.getBufferAsString() );
    GAIACRTN_ASSERT_EQUAL( _strExpected , _test.getBufferAsString() );
  }

private:
  TtrUtWriterWrappedTxt &_test;
  EString _strParagraph;
  EString _strExpected;
};


// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// Class TtrUtWriterWrappedTxt
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------


TtrUtWriterWrappedTxt::TtrUtWriterWrappedTxt() : 
    CppUnit::TestCase()
{
}


TtrUtWriterWrappedTxt::~TtrUtWriterWrappedTxt()
{
}


void 
TtrUtWriterWrappedTxt::setUp()
{
  _pBuffer = new EBuffer();
  _pBuffer->open( EIO_ReadWrite );
  _pStream = new ETextStream( _pBuffer );
  _strEol = _pStream->getEol();
  _pSettingTab = new TtrGlobalSetting();
  _pSettingTab->setWriterWrappingCol( 40 );
  _pWriter = new TtrWriterWrappedTxt( *_pStream, *_pSettingTab );
}


void 
TtrUtWriterWrappedTxt::tearDown()
{
  delete _pWriter;
  delete _pSettingTab;
  delete _pStream;
  delete _pBuffer;
}


void 
TtrUtWriterWrappedTxt::testFindBreakPos()
{
             //  0000000000111111111122222222223333333333444444444455555555556666666666777777777788888888889999999999 
             //  0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789
  EString str1( "She shook her head, scrubbing at her eyes, and washing away the tears forcefully." );
  static int anBreak1[] = { 3,1, 9,1, 13,1, 19,1, 29,1, 32,1, 36,1, 42,1, 46,1, 54,1, 59,1, 63,1, 69,1, 81,0, -1,1 };
  checkFindBreakPos( str1, anBreak1 );

             //  0000000000111111111122222222223333333333444444444455555555556666666666777777777788888888889999999999 
             //  0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789
  EString str2( "  She was going to go to outside,  and she was going to have a good day, and... and...  and..." );
  static int anBreak2[] = { 5,1, 9,1, 15,1, 18,1, 21,1, 24,1, 33,2, 38,1, 42,1, 46,1, 52,1, 55,1, 60,1, 62,1, 67,1, 72,1, 79,1, 86,2, 94,0, -1,1 };
  checkFindBreakPos( str2, anBreak2 );

  EString strEmpty;
  static int anBreakEmpty[] = { 0,0, -1,-1 };
  checkFindBreakPos( strEmpty, anBreakEmpty );

             //    0 12 34 567890
  EString strTab( "\t*\t*\t*" );
  static int anBreakTab[] = { 2,1, 4,1, 6,0, -1,-1 };
  checkFindBreakPos( strTab, anBreakTab );
}


void 
TtrUtWriterWrappedTxt::checkFindBreakPos( const EString &str, 
                                          int breakPos[] )
{
  TtrWriterWrappedTxt::ArrayBreakPos breaks;
  _pWriter->findBreakPos( str, breaks );

  int *pBreak = breakPos;
  int nIdx = 0;
  do
  {
    int nBreakPos = *pBreak++;
    int nBreakLength = *pBreak++;
    if ( nBreakPos == -1 )
      break;

    GAIACRTN_ASSERT( nBreakLength >= 0 );
    GAIACRTN_ASSERT_EQUAL( nBreakPos, breaks[nIdx]._nPos );
    GAIACRTN_ASSERT_EQUAL( nBreakLength, breaks[nIdx]._nLength );

    ++nIdx;
  }
  while ( true );
}


void 
TtrUtWriterWrappedTxt::testBreakParaInLine()
{
             //  0000000000111111111122222222223333333333444444444455555555556666666666777777777788888888889999999999 
             //  0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789
  Strings lines;
  lines.append( "She shook her head, scrubbing at her" );
  lines.append( "eyes, and washing away the tears" );
  lines.append( "forcefully.  She was going to go to" );
  lines.append( "outside,  and she was going to have a" );
  lines.append( "good day, and... and...  and..." );
  checkBreakParaInLine( lines, 40, 0 );
}


void 
TtrUtWriterWrappedTxt::testBreakTooLongParaInLine()
{
  Strings lines;
  lines.append( "Imreallyinahurry," );
  lines.append( "andidonthavethetimetostay" );
  checkBreakParaInLine( lines, 10, 0 );
}


void 
TtrUtWriterWrappedTxt::testBreakShortParaInLine()
{
  Strings lines;
  lines.append( "Nice day." );
  checkBreakParaInLine( lines, 40, 0 );
}


void 
TtrUtWriterWrappedTxt::testBreakEmptyParaInLine()
{
  Strings lines;
  lines.append( "" );
  checkBreakParaInLine( lines, 40, 0 );
}


void
TtrUtWriterWrappedTxt::checkBreakParaInLine( const Strings &linesPredicted,
                                             int nLineLength,
                                             int nLeftColPos ) const
{
  EString strParagraph = collateLines( linesPredicted );
  TtrWriterWrappedTxt::Strings linesRealized;
  _pWriter->breakParagraph( strParagraph, linesRealized, nLineLength, nLeftColPos );

  GAIACRTN_ASSERT_EQUAL( linesPredicted.count(), linesRealized.count() );
  for ( int nIdx =0; nIdx < linesPredicted.count(); ++nIdx )
  {
    const EString &strPredicted = linesPredicted[nIdx];
    const EString &strRealized = linesRealized[nIdx];
//    TRACE( "Pred: %s\n", (LPCTSTR)strPredicted );
//    TRACE( "Real: %s\n", (LPCTSTR)strRealized );
    GAIACRTN_ASSERT_EQUAL( strPredicted, strRealized );
  }
}


EString 
TtrUtWriterWrappedTxt::collateLines( const Strings &astrLine ) const
{
  EString strParagraph;

  for ( int nIdx = 0; nIdx < astrLine.count(); ++nIdx )
  {
    if ( nIdx > 0 )
      strParagraph += " ";
    strParagraph += astrLine[nIdx];
  }

  return strParagraph;
}


void 
TtrUtWriterWrappedTxt::testComputeRightMarginPos()
{
  EString strEmpty;
  GAIACRTN_ASSERT_EQUAL( 5, _pWriter->computeRightMarginPos( strEmpty, 0,0, 5 ) );

  EString strBasic( "A simple string." );
  GAIACRTN_ASSERT_EQUAL( strBasic.length(), 
      _pWriter->computeRightMarginPos( strBasic, 0, strBasic.length(), 0 ) );
  GAIACRTN_ASSERT_EQUAL( 6, 
      _pWriter->computeRightMarginPos( strBasic, 2, 8, 0 ) );

                    //0 8  16 24
  EString strTabbed( "\t*\t*\t*" );
  GAIACRTN_ASSERT_EQUAL( 25, 
      _pWriter->computeRightMarginPos( strTabbed, 0, strTabbed.length(), 7 ) );
}


void 
TtrUtWriterWrappedTxt::testReplaceTabWithSpace()
{
  GAIACRTN_ASSERT_EQUAL( "", _pWriter->replaceTabWithSpace( "" ));

  EString strTabbedStar( "*\t*\t*" );
  GAIACRTN_ASSERT_EQUAL( "* * *", _pWriter->replaceTabWithSpace( strTabbedStar ) );

  EString strSimpleText( "  sometext" );
  GAIACRTN_ASSERT_EQUAL( strSimpleText,
                         _pWriter->replaceTabWithSpace( strSimpleText ) );

  EString strTabIndented( "\tNew" );
  GAIACRTN_ASSERT_EQUAL( " New", _pWriter->replaceTabWithSpace( strTabIndented ) );
}


EString 
TtrUtWriterWrappedTxt::getBufferAsString()
{
  EString str;
  str.copy( reinterpret_cast<const char *>(_pBuffer->buffer()), 
            _pBuffer->size() );
  return str;
}


void 
TtrUtWriterWrappedTxt::testVisitCenteredPara()
{
  WrapChecker checker( *this );
  checker.addLine( "She shook her head, scrubbing at her" );  //+4
  checker.addLine( "eyes, and washing away the tears" );      //+8
  checker.addLine( "forcefully.  She was going to go to" );   //+5
  checker.addLine( "outside,  and she was going to have a" ); //+3
  checker.addLine( "good day, and... and...  and..." );       //+9

                    //  0000000000111111111122222222223333333333444444444455555555556666666666777777777788888888889999999999 
                    //  0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789
  checker.addExpected( "  She shook her head, scrubbing at her" ); //+3 (-1:indent)
  checker.addExpected( "    eyes, and washing away the tears" );   //+8
  checker.addExpected( "  forcefully.  She was going to go to" );  //+5
  checker.addExpected( " outside,  and she was going to have a" ); //+3
  checker.addExpected( "    good day, and... and...  and..." );    //+9

  checker.check( 1, "", true );
}


void 
TtrUtWriterWrappedTxt::testVisitCenteredParaWithPrefix()
{
  WrapChecker checker( *this );
  checker.addLine( "She shook her head, scrubbing at her" );  //+1
  checker.addLine( "eyes, and washing away the tears" );      //+5
  checker.addLine( "forcefully.  She was going to go to" );   //+2
  checker.addLine( "outside,  and she was going to have a" ); //+0
  checker.addLine( "good day, and... and...  and..." );       //+6

                    //  0000000000111111111122222222223333333333444444444455555555556666666666777777777788888888889999999999 
                    //  0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789
  checker.addExpected( "> > She shook her head, scrubbing at her" ); //+0  (-1:indent)
  checker.addExpected( "> >  eyes, and washing away the tears" );    //+5
  checker.addExpected( "> > forcefully.  She was going to go to" );  //+2
  checker.addExpected( "> >outside,  and she was going to have a" ); //+0
  checker.addExpected( "> >   good day, and... and...  and..." );    //+6

  checker.check( 1, "> >", true );
}


void 
TtrUtWriterWrappedTxt::testVisitLeftAlignedPara()
{
  WrapChecker checker( *this );
  checker.addLine(         "She shook her head, scrubbing at" );
  checker.addLine( "her eyes, and washing away the tears" );
  checker.addLine( "forcefully.  She was going to go to" );
  checker.addLine( "outside,  and she was going to have a" );
  checker.addLine( "good day, and... and...  and..." );

                    //  0000000000111111111122222222223333333333444444444455555555556666666666777777777788888888889999999999 
                    //  0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789
  checker.addExpected(       "\tShe shook her head, scrubbing at" );
  checker.addExpected( "her eyes, and washing away the tears" );
  checker.addExpected( "forcefully.  She was going to go to" );
  checker.addExpected( "outside,  and she was going to have a" );
  checker.addExpected( "good day, and... and...  and..." );

  checker.check( 1, "", false );
}


void 
TtrUtWriterWrappedTxt::testVisitLeftAlignedParaWithPrefix()
{
  WrapChecker checker( *this );
  checker.addLine( "She shook her head, scrubbing at" );
  checker.addLine( "her eyes, and washing away the tears" );
  checker.addLine( "forcefully.  She was going to go to" );
  checker.addLine( "outside,  and she was going to have a" );
  checker.addLine( "good day, and... and...  and..." );

                    //  0000000000111111111122222222223333333333444444444455555555556666666666777777777788888888889999999999 
                    //  0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789
  checker.addExpected(    "> >\tShe shook her head, scrubbing at" );
  checker.addExpected( "> >her eyes, and washing away the tears" );
  checker.addExpected( "> >forcefully.  She was going to go to" );
  checker.addExpected( "> >outside,  and she was going to have a" );
  checker.addExpected( "> >good day, and... and...  and..." );

  checker.check( 1, "> >", false );
}


void 
TtrUtWriterWrappedTxt::testBreakVeryLongBlankLine()
{
  Strings lines;
  lines.append( "      -----" );
  checkBreakParaInLine( lines, 4, 0 );
}
