// //////////////////////////////////////////////////////////////////////////
// Header file TtrUtWriterWrappedTxt.h for class TtrUtWriterWrappedTxt
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2001/05/06
// //////////////////////////////////////////////////////////////////////////
#ifndef TTRUTWRITERWRAPPEDTXT_H
#define TTRUTWRITERWRAPPEDTXT_H

#include "cppunit.h"

class TtrWriterWrappedTxt;
class TtrGlobalSetting;


/*! \class TtrUtWriterWrappedTxt
 * \brief This class represents a test case for TtrWriterWrappedTxt.
 */
class TtrUtWriterWrappedTxt : public CppUnit::TestCase
{
  CPPUNIT_TEST_SUITE( TtrUtWriterWrappedTxt );
  CPPUNIT_TEST( testFindBreakPos );
  CPPUNIT_TEST( testBreakParaInLine );
  CPPUNIT_TEST( testBreakTooLongParaInLine );
  CPPUNIT_TEST( testBreakShortParaInLine );
  CPPUNIT_TEST( testBreakEmptyParaInLine );
  CPPUNIT_TEST( testComputeRightMarginPos );
  CPPUNIT_TEST( testReplaceTabWithSpace );
  CPPUNIT_TEST( testVisitCenteredPara );
  CPPUNIT_TEST( testVisitCenteredParaWithPrefix );
  CPPUNIT_TEST( testVisitLeftAlignedPara );
  CPPUNIT_TEST( testVisitLeftAlignedParaWithPrefix );
  CPPUNIT_TEST( testBreakVeryLongBlankLine );
  CPPUNIT_TEST_SUITE_END();

private:
  class WrapChecker;
  friend class WrapChecker;

public:
  /*! Constructs a TtrUtWriterWrappedTxt object.
   */
  TtrUtWriterWrappedTxt();

  /*! Destructor.
   */
  virtual ~TtrUtWriterWrappedTxt();

  void setUp();
  void tearDown();

  void testFindBreakPos();
  void testBreakParaInLine();
  void testBreakTooLongParaInLine();
  void testBreakShortParaInLine();
  void testBreakEmptyParaInLine();

  void testComputeRightMarginPos();
  void testReplaceTabWithSpace();

  void testVisitCenteredPara();
  void testVisitCenteredParaWithPrefix();
  void testVisitLeftAlignedPara();
  void testVisitLeftAlignedParaWithPrefix();
  void testBreakVeryLongBlankLine();

private:
  typedef EObjDynArrayT<EString> Strings;

private:
  /// Prevents the use of the copy constructor.
  TtrUtWriterWrappedTxt( const TtrUtWriterWrappedTxt &copy );

  /// Prevents the use of the copy operator.
  void operator =( const TtrUtWriterWrappedTxt &copy );

  void checkFindBreakPos( const EString &str, 
                          int breakPos[] );

  EString collateLines( const Strings &astrLine ) const;

  void checkBreakParaInLine( const Strings &linesPredicted,
                             int nLineLength,
                             int nLeftColPos ) const;

  EString getBufferAsString();
private:
  TtrWriterWrappedTxt *_pWriter;
  TtrGlobalSetting *_pSettingTab;
  EBuffer *_pBuffer;
  ETextStream *_pStream;
  EString _strEol;
};



// Inlines methods for TtrUtWriterWrappedTxt:
// ------------------------------------------



#endif  // TTRUTWRITERWRAPPEDTXT_H
